<?php

/*
 * @author Targetbay
 * @copyright 2016 Sathishkumar Mariappan <sathishkumar.m@innoppl.com>
 * @package Targetbay_Tracking
 */

class Targetbay_Tracking_Block_Product_Inventary extends Mage_Core_Block_Template
{
    /**
     * Rendered the tracking review template if tracking enabled
     *
     * @see Mage_Core_Block_Template::_construct()
     */
    public function _construct()
    {
        parent::_construct();
        if ((int)Mage::helper('tracking')->getBackOrderStatus()) {
            $this->setTemplate('tracking/product/inventary.phtml');
        }
    }

    /**
     * Get the current product
     * @return Mage_Catalog_Model_Product
     */
    public function getProduct()
    {
        return Mage::registry('current_product');
    }

    /**
     * @return string
     */
    public function getBackOrderStatus()
    {
        $html = '';
        if ($this->getInventoryStatus() == 1) {
            $html .= '<div id="tb-backinstock"></div>';
        }

        return $html;
    }

    /**
     * @return bool|int
     */
    public function getInventoryStatus()
    {
        $product = $this->getProduct();
        $currentProductId = $product->getId();
        $currentProductTypeId = $product->getTypeId();
        $status = 0;
        switch ($currentProductTypeId) {
            case Mage_Catalog_Model_Product_Type::TYPE_CONFIGURABLE:
                $childProducts = Mage::getModel('catalog/product_type_configurable')
                                    ->getUsedProducts(null, $product);
                foreach ($childProducts as $child) {
                    if ($child->getStockItem()->getIsInStock() == 0) {
                        $status = true;
                    }
                }
                break;
            case Mage_Catalog_Model_Product_Type::TYPE_GROUPED:
                $childProducts = $product->getTypeInstance(true)->getAssociatedProducts($product);
                foreach ($childProducts as $child) {
                    if ($child->getStockItem()->getIsInStock() == 0) {
                        $status = true;
                    }
                }
                break;
            case Mage_Catalog_Model_Product_Type::TYPE_BUNDLE:
                $product = Mage::getModel('catalog/product')->load($currentProductId);
                $selectionCollection = $product->getTypeInstance(true)
                                ->getSelectionsCollection($product->getTypeInstance(true)->getOptionsIds($product), $product);

                //$i = 0;
                foreach ($selectionCollection as $child) {
                    if ($child->getStockItem()->getIsInStock() == 0) {
                        $status = true;
                    }
                }
                break;
            case Mage_Catalog_Model_Product_Type::TYPE_SIMPLE:
                if ($product->getStockItem()->getIsInStock() == 0) {
                    $status =  true;
                }
                break;
            default:
                break;
        }

        return $status;
    }
}
