<?php

/*
 * @author TargetBay
 * @copyright 2016 Sathishkumar Mariappan <sathishkumar.m@innoppl.com>
 * @package Targetbay_Tracking
 */

class Targetbay_Tracking_Block_Product_Review extends Mage_Core_Block_Template
{
    /**
     * Rendered the tracking review template if tracking enabled
     *
     * @see Mage_Core_Block_Template::_construct()
     */
    public function _construct()
    {
        parent::_construct();
        if (Mage::helper('tracking')->trackingEnabled()) {
            $this->setTemplate('tracking/product/review.phtml');
        }
    }

    /**
     * Get the current product
     * @return Mage_Catalog_Model_Product
     */
    public function getProduct()
    {
        return Mage::registry('current_product');
    }

    /**
     * @return mixed
     */
    public function getEscapedProductName()
    {
        return $this->escapeHtml($this->getProduct()->getName());
    }

    /**
     * @return string
     */
    public function getEscapedDescription()
    {
        $product = $this->getProduct();

        if (!$product) {
            return '';
        }

        $string = $product->getDescription();

        return strip_tags(html_entity_decode($string));
    }

    /**
     * @return string
     */
    public function getSku()
    {
        $product = $this->getProduct();

        if (!$product) {
            return '';
        }

        return $product->getSku();
    }

    /**
     * @return string
     */
    public function getImageUrl()
    {
        $product = $this->getProduct();

        if (!$product) {
            return '';
        }

        return  Mage::helper('catalog/image')->init($product, 'image');
    }


    /**
     * To get the Grouped product price
     * @return FLOAT|NULL
     */
    public function getProductPrice()
    {
        $product = $this->getProduct();
        if (!$product) {
            return '';
        }
        switch ($product->getTypeId()) {
            case 'grouped':
                $_groupedProductChildPrices = array();
                $_childProductIds = $product->getTypeInstance()->getChildrenIds($product->getId());
                foreach ($_childProductIds as $_ids) {
                    foreach ($_ids as $_id) {
                        $_childProduct = Mage::getModel('catalog/product')->load($_id);
                        $_groupedProductChildPrices[] = $_childProduct->getFinalPrice();
                        if($_childProduct->getTierPrice() != NULL) {
                            foreach($_childProduct->getTierPrice() as $tier){
                                $_groupedProductChildPrices[]= $tier['price'];
                            }
                        }
                    }
                }
                $FinalPrice = min($_groupedProductChildPrices);
                break;

            default:
                 $FinalPrice = $product->getFinalPrice();
                break;
        }
        return $FinalPrice;
    }
}
