<?php

/*
 * @author TargetBay
 * @copyright 2016 Sathishkumar Mariappan <sathishkumar.m@innoppl.com>
 * @package Targetbay_Tracking
 */

class Targetbay_Tracking_Block_Tracking extends Mage_Core_Block_Template
{
    /**
     * Rendered the tracking template if tracking enabled
     *
     * @see Mage_Core_Block_Template::_construct()
     */
    public function _construct()
    {
        parent::_construct();
        if (Mage::helper('tracking')->trackingEnabled()) {
            $this->setTemplate('tracking/tracking.phtml');
        }
    }

    /**
     * Get the current product
     * @return Mage_Catalog_Model_Product
     */
    public function getProduct()
    {
        return Mage::registry('current_product');
    }

    /**
     * @return int
     */
    public function getProductStockInfo()
    {
        $_product = $this->getProduct();
        $controllerName = Mage::app()->getRequest()->getControllerName();
        $moduleName = Mage::app()->getRequest()->getModuleName();
        $backOrderStatus = Mage::helper('tracking')->getBackOrderStatus();

        if ($moduleName === 'catalog' && $controllerName === 'product' && $backOrderStatus == 1) {
            if ($this->getInventoryStatus() == 1) {
                return Mage::helper('tracking/inventary')->getProductInfo();
            } else {
                return 0;
            }
        } else {
            return 0;
        }
    }

    /**
     * @return bool|int
     */
    public function getStockAvailability()
    {
        $_product = $this->getProduct();
        $controllerName = Mage::app()->getRequest()->getControllerName();
        $moduleName = Mage::app()->getRequest()->getModuleName();
        $backOrderStatus = Mage::helper('tracking')->getBackOrderStatus();

        if ($moduleName === 'catalog' && $controllerName === 'product' && $backOrderStatus == 1) {
            return $this->getInventoryStatus();
        }

        return 0;
    }

    /**
     * @return bool|int
     */
    public function getInventoryStatus()
    {
        $product = $this->getProduct();
        $currentProductId = $product->getId();
        $currentProductTypeId = $product->getTypeId();
        $status = 0;

        switch ($currentProductTypeId) {
            case Mage_Catalog_Model_Product_Type::TYPE_CONFIGURABLE:
                $childProducts = Mage::getModel('catalog/product_type_configurable')
                                    ->getUsedProducts(null, $product);
                foreach ($childProducts as $child) {
                    if ($child->getStockItem()->getIsInStock() == 0) {
                        $status = true;
                    }
                }
                break;
            case Mage_Catalog_Model_Product_Type::TYPE_GROUPED:
                $childProducts = $product->getTypeInstance(true)->getAssociatedProducts($product);
                foreach ($childProducts as $child) {
                    if ($child->getStockItem()->getIsInStock() == 0) {
                        $status = true;
                    }
                }
                break;
            case Mage_Catalog_Model_Product_Type::TYPE_BUNDLE:
                $product = Mage::getModel('catalog/product')->load($currentProductId);
                $selectionCollection = $product->getTypeInstance(true)
                                ->getSelectionsCollection($product->getTypeInstance(true)->getOptionsIds($product), $product);

                //$i = 0;
                foreach ($selectionCollection as $child) {
                    if ($child->getStockItem()->getIsInStock() == 0) {
                        $status = true;
                    }
                }
                break;
            case Mage_Catalog_Model_Product_Type::TYPE_SIMPLE:
                if ($product->getStockItem()->getIsInStock() == 0) {
                    $status =  true;
                }
                break;
            default:
                break;
        }
        return $status;
    }

    /**
     * @return mixed
     */
    public function getVisitorId()
    {
        $trackingType = Mage::helper('tracking')->getTrackingType();
        $customer = Mage::getSingleton('customer/session');

        if ($trackingType == 1) {
            $visitorId = Mage::helper('tracking')->getFetchPointsByKey('_usid');
            $userId = $customer->isLoggedIn() ? $customer->getCustomer()->getId() : $visitorId;
        } else {
            $visitorId = Mage::getSingleton('core/session')->getTrackingSessionId();
            $userId = $customer->isLoggedIn() ? $customer->getCustomer()->getId() : $visitorId;
        }

        return $userId;
    }

    /*
     * Get Parent product Id of child product to map product reviews.
     */
    public function getProductId()
    {
        $product = $this->getProduct();
        $productParentId = Mage::getModel('tracking/tracking')->getFirstParentProductId();

        if ($product->getTypeId() == Mage_Catalog_Model_Product_Type::TYPE_SIMPLE
            && $productParentId > 0) {
            $productId = $productParentId;
        } else {
            $productId = $product->getId();
        }

        return $productId;
    }
}
