<?php

/*
 * @author TargetBay
 * @copyright 2016 Sathishkumar Mariappan <sathishkumar.m@innoppl.com>
 * @package Targetbay_Tracking
 */

class Targetbay_Tracking_Helper_Data extends Mage_Core_Helper_Abstract
{
    public $cookie;
    public $date;
    public $storeId;
    public $storeIdForAdminEvents = NULL;

    const ANONYMOUS_USER = 'anonymous';
    // page types
    const ALL_PAGES = 'all';
    const PAGE_VISIT = 'page-visit';
    const PRODUCT_VIEW = 'product-view';
    const CATEGORY_VIEW = 'category-view';
    const DELETE_PRODUCT = 'delete-product';
    const UPDATE_PRODUCT = 'update-product';
    const ADD_PRODUCT = 'add-product';
    const CREATE_ACCOUNT = 'create-account';
    const ADMIN_ACTIVATE_ACCOUNT = 'admin-activate-customer-account';
    const ADMIN_DELETE_ACCOUNT = 'admin-delete-customer-account';
    const LOGIN = 'login';
    const LOGOUT = 'logout';
    const ADDTOCART = 'add-to-cart';
    const REMOVECART = 'remove-to-cart';
    const UPDATECART = 'update-cart';
    const ORDER_ITEMS = 'ordered-items';
    const BILLING = 'billing';
    const SHIPPING = 'shipping';
    const PAGE_REFERRAL = 'referrer';
    const CHECKOUT = 'checkout';
    const CATALOG_SEARCH = 'searched';
    const WISHLIST = 'wishlist';
    const UPDATE_WISHLIST = 'update-wishlist';
    const REMOVE_WISHLIST = 'remove-wishlist';
    const ONESTEPCHECKOUT_ADDRESS = 'onestepcheckout';
    const CART_INDEX = 'checkout-cart';
    const SUBSCRIBE_CUSTOMER = 'user-subscribe';
    const CUSTOMER_ADDRESS = 'change-user-address';
    const CUSTOMER_ACCOUNT = 'change-user-account-info';
    const CREATE_ADDRESS = 'new';
    const UPDATE_ADDRESS = 'edit';
    const RATINGS_STATS = 'ratings-stats';
    const QUESTION_STATS = 'qa-stats';
    const CATEGORY_STATS = 'cat-stats';
    const PRODUCT_STATUS_MASSUPDATE = 'product-status-massupdate';
    const PRODUCT_UPDATE_IMPORT = 'product-update-importcsv';

    // subscription status
    const STATUS_SUBSCRIBED = 1;
    const STATUS_NOT_ACTIVE = 2;
    const STATUS_UNSUBSCRIBED = 3;
    const STATUS_UNCONFIRMED = 4;

    // product stock status
    const IN_STOCK = 'in-stock';
    const OUT_OF_STOCK = 'out-stock';

    // order fullfillment process
    const ORDER_SHIPMENT = 'shipment';
    const ORDER_INVOICE = 'invoice';
    const ORDER_REFUND = 'creditmemo';
    const ORDER_STATUS = 'order-status';

    // ToDo: unused constants.
    const HOST_STAGE = 'https://stage.targetbay.com/api/v1/webhooks/';
    const HOST_LIVE = 'https://app.targetbay.com/api/v1/webhooks/';
    const HOST_DEV = 'https://dev.targetbay.com/api/v1/webhooks/';

    // ToDo: unused constants.
    const API_STAGE = 'stage';
    const API_LIVE = 'app';
    const API_DEV = 'dev';

    const DEFAULT_TIMEOUT = 30;

    /**
     * Initialize object
     */
    public function __construct()
    {
        $this->cookie = Mage::getModel('core/cookie');
        $this->date = Mage::getModel('core/date');
        $this->storeId = (Mage::app()->getStore()->getId() == 0) ? $this->getStoreIdForAdminEvents() : Mage::app()->getStore()->getId();
        // $this->storeId = Mage::app()->getStore()->getId();
    }

    /**
     * Get Store Id for Admin Events; if store id = 0
     *
     * @return INT
     */
    public function getStoreIdForAdminEvents()
    {
        if($this->storeIdForAdminEvents != NULL){
        	return $this->storeIdForAdminEvents;
        }
        return $this->TBenabledStoreId();
    }

    /**
     * Set Store Id for Admin Events by Observer
     *
     * @return INT
     */
    public function setStoreIdForAdminEvents($EventStoreId = NULL)
    {
    	if($EventStoreId && Mage::getStoreConfig('targetbay_tracking/tracking_groups/enabled', $EventStoreId)){
    		$this->storeIdForAdminEvents = $EventStoreId;
        	return;
    	}
    }

    /**
     * Get TB enabled Store Id
     *
     * @return INT
     */
    public function TBenabledStoreId()
    {
        $TBconfigPath =  "targetbay_tracking/tracking_groups/enabled";

        $TbScopeData = Mage::getModel('core/config_data')->getCollection()
                    ->addFieldToFilter('path',$TBconfigPath)
                    ->addFieldToFilter('value',1)->getFirstItem();
        //$store_code = $TbScopeData->getScope();
        $store_id = $TbScopeData->getScopeId();
        return $store_id;
    }


    /**
     * Check module is enabled or not
     *
     * @return boolean
     */
    public function trackingEnabled()
    {
       return (bool) Mage::getStoreConfig('targetbay_tracking/tracking_groups/enabled', $this->storeId);
    }

    /**
     * Get TargetBay end point and tokens.
     *
     * @param $page
     * @param $webId
     * @return mixed
     */
    public function getHost($page, $webId = null)
    {
        if($webId === null) {
            $webId = $this->storeId;
        }

        $apiStatus = Mage::getStoreConfig('targetbay_tracking/tracking_groups/api_status', $webId);
        $config = Mage::app()->getStore($webId);

        // Get Apikey
        $apiKey = $config->getConfig('targetbay_tracking/tracking_groups/api_token');
        if(!$apiKey){
             $apiKey = Mage::getStoreConfig('targetbay_tracking/tracking_groups/api_token', $webId);
        }

        // Get apisecret
        $apiSecret = $config->getConfig('targetbay_tracking/tracking_groups/api_index');
        if(!$apiSecret){
             $apiSecret = Mage::getStoreConfig('targetbay_tracking/tracking_groups/api_index', $webId);
        }
        
        return 'https://' . $apiStatus . '.targetbay.com/api/v1/magento/' . $page . '?_t=' . base64_encode('_a=' . $apiKey .'&_i=' . $apiSecret);
    }

    /**
     * Get TargetBay end point.
     *
     * @return mixed
     */
    public function getHostname()
    {
        return Mage::getStoreConfig('targetbay_tracking/tracking_groups/hostname', $this->storeId);
    }

    /**
     * Get TargetBay Token.
     *
     * @param $websiteId
     *
     * @return mixed
     */
    public function getApiToken($websiteId)
    {
       $checkWebsiteApiToken = Mage::app()->getStore($websiteId)
            ->getConfig('targetbay_tracking/tracking_groups/api_token');
        if($checkWebsiteApiToken){
                return $checkWebsiteApiToken;
        }else{
                return Mage::getStoreConfig('targetbay_tracking/tracking_groups/api_token', $this->storeId);
        }
    }

    /**
     * Get TargetBay Index.
     *
     * @param $websiteId
     *
     * @return mixed
     */
    public function getApiIndex($websiteId)
    {
       // Before we have stored the data based on website. Now changed to store.
        $checkWebsiteApiIndex = Mage::app()->getStore($websiteId)
            ->getConfig('targetbay_tracking/tracking_groups/api_index');
        if($checkWebsiteApiIndex){
             return $checkWebsiteApiIndex;
        }else{
            return Mage::getStoreConfig('targetbay_tracking/tracking_groups/api_index', $this->storeId);
        }
    }

    /**
     * Get getApiStatus
     *
     * @return mixed
     */
    public function getApiStatus()
    {
        return Mage::getStoreConfig('targetbay_tracking/tracking_groups/api_status', $this->storeId);
    }

    /**
     * Get the Session Tracking Js.
     *
     * @return mixed
     */
    public function getReviewPageSize()
    {
        $reviewSize = Mage::getStoreConfig('targetbay_tracking/tracking_groups/reviews_per_page', $this->storeId);

        if (! $reviewSize) {
            $reviewSize = 10;
        }

        return $reviewSize;
    }

    /**
     * Get the email status.
     *
     * @return mixed
     */
    public function getEmailStatus()
    {
        $emailStatus = Mage::getStoreConfig('targetbay_tracking/tracking_groups/disable_email', $this->storeId);
        if (! $emailStatus) {
            $emailStatus = 1;
        }

        return $emailStatus;
    }

    /**
     * Get TargetBay tracking type
     *
     * @return mixed
     */
    public function getTrackingType()
    {
        return Mage::getStoreConfig('targetbay_tracking/tracking_groups/tracking_type', $this->storeId);
    }

     /**
     * Get Persistent enable check
     *
     * @return mixed
     */
    public function getPersistentCartCheck()
    {
        return Mage::getStoreConfig('persistent/options/enabled', $this->storeId);
    }


    /**
     * Get TargetBay richsnippet type
     *
     * @return mixed
     */
    public function getRichSnippetType()
    {
        return Mage::getStoreConfig('targetbay_tracking/tracking_google_snippets/richsnippets_type', $this->storeId);
    }

    /**
     * Get TargetBay site review
     *
     * @return mixed
     */
    public function getGoogleSiteReviewSnippet()
    {
        return Mage::getStoreConfig('targetbay_tracking/tracking_google_snippets/site_review', $this->storeId);
    }

    /**
    * Get TargetBay site review for Home page
    *
    * @return mixed
    */
    public function getGoogleSiteReviewSnippetHome()
    {
        return Mage::getStoreConfig('targetbay_tracking/tracking_google_snippets/site_reviewhome', $this->storeId);
    }

    /**
     * Get TargetBay product review
     *
     * @return mixed
     */
    public function getGoogleProductReviewSnippet()
    {
        return Mage::getStoreConfig('targetbay_tracking/tracking_google_snippets/product_review', $this->storeId);
    }

    /**
     * Get TargetBay question review
     *
     * @return mixed
     */
    public function getGoogleQuestionReviewSnippet()
    {
        return Mage::getStoreConfig('targetbay_tracking/tracking_google_snippets/question_review', $this->storeId);
    }

    /**
     * Get TargetBay question review
     *
     * @return mixed
     */
    public function getGoogleProductReviewFeed()
    {
        return Mage::getStoreConfig('targetbay_tracking/tracking_google_review/feed_enable', $this->storeId);
    }

    /**
     * Print the log or not
     *
     * @return mixed
     */
    public function logEnabled()
    {
        return Mage::getStoreConfig('targetbay_tracking/tracking_groups/debug', $this->storeId);
    }

    /**
     * Get the log file name from configurations
     */
    public function getLogFileName()
    {
        return Mage::getStoreConfig('targetbay_tracking/tracking_groups/debug_file', $this->storeId);
    }

    /**
     * Check backorders is enabled or not
     *
     * @return boolean
     */
    public function getBackOrderStatus()
    {
        return (bool) Mage::getStoreConfig('targetbay_tracking/tracking_groups/manage_stock', $this->storeId);
    }

    /**
     * Check Recommendation is enabled or not
     *
     * @return boolean
     */
    public function getRecommendationStatus()
    {
        return (bool) Mage::getStoreConfig('targetbay_tracking/tracking_product_recommendation/enable_recommendation', $this->storeId);
    }

    /**
     * Check Instagram is enabled or not
     *
     * @return boolean
     */
    public function getInstagramStatus()
    {
        return (bool) Mage::getStoreConfig('targetbay_tracking/tracking_instagram/enable_instagram', $this->storeId);
    }

    /**
     * Check Instagram is enabled or not for home page
     *
     * @return boolean
     */
    public function getInstagramHomeStatus()
    {
        return (bool) Mage::getStoreConfig('targetbay_tracking/tracking_instagram/homepage', $this->storeId);
    }

    /**
     * Check Instagram is enabled or not product page
     *
     * @return boolean
     */
    public function getInstagramProductStatus()
    {
        return (bool) Mage::getStoreConfig('targetbay_tracking/tracking_instagram/product_page', $this->storeId);
    }

    /**
     * Get the available pages from configurations
     *
     * @return array
     */
    public function availablePageTypes()
    {
        $types = (string) Mage::getStoreConfig('targetbay_tracking/tracking_groups/page_types', $this->storeId);
        return explode(',', $types);
    }

    /**
     * Get Tracking Code
     *
     * @return mixed
     */
    public function getTrackingScript()
    {
        return Mage::getStoreConfig('targetbay_tracking/tracking_groups/tracking_script', $this->storeId);
    }

    /**
     * Get Google Customer Reviews badge status.
     *
     * @return mixed
     */
    public function getGoogleCustomerReviewBadge()
    {
        return (bool) Mage::getStoreConfig('targetbay_tracking/google_customer_reviews/badge_enabled', $this->storeId);
    }

    /**
     * Check Recommendation is enabled or not
     *
     * @return boolean
     */
    public function getGoogleMerchantId()
    {
        return Mage::getStoreConfig('targetbay_tracking/google_customer_reviews/merchant_id', $this->storeId);
    }

    /**
     * Check Recommendation is enabled or not
     *
     * @return boolean
     */
    public function getGoogleBadgePosition()
    {
        return Mage::getStoreConfig('targetbay_tracking/google_customer_reviews/badge_position', $this->storeId);
    }

    /**
     * Check the page configurations
     *
     * @param $pageType
     * @return boolean
     */
    public function canTrackPages($pageType)
    {
        if (!$this->trackingEnabled()) {
            $this->debug('Tracking Module is not enabled. Please enable a Module');
            return false;
        }

        $availablePages = $this->availablePageTypes();
        if (in_array(self::ALL_PAGES, $availablePages, true)) {
            return true;
        }

        if (!in_array($pageType, $availablePages, true)) {
            $this->debug("'$pageType'" . 'page is not enabled.');
            return false;
        }

        return true;
    }

    /**
     * Get the coupon code
     *
     * @return mixed
     */
    public function getProductSkuToRedeem()
    {
        return Mage::getStoreConfig('targetbay_tracking/tracking_coupon/product_sku', $this->storeId);
    }

    /**
     * Get the customer data based on the action
     *
     * @param $customer
     * @param $action
     * @return array
     */
    public function getCustomerData($customer, $action)
    {
        try {
            switch ($action) {
                case self::LOGIN:
                    $data = $this->getCustomerSessionId($customer);
                    $data['login_date'] = $this->date->date('Y-m-d');
                    $data['firstname'] = $customer->getFirstname();
                    $data['lastname'] = $customer->getLastname();
                    break;
                case self::LOGOUT:
                    $data['session_id'] = Mage::getSingleton('core/session')->getCustomerSessionId();
                    $data['logout_date'] = $this->date->date('Y-m-d');
                    Mage::getSingleton('core/session')->unsTrackingSessionId();
                    Mage::getSingleton('core/session')->unsCustomerSessionId();
                    break;
                case self::CREATE_ACCOUNT:
                    $data = $this->getCustomerSessionId($customer);
                    $data['firstname'] = $customer->getFirstname();
                    $data['lastname'] = $customer->getLastname();
                    $customerGroupId  = $customer->getGroupId();
                    $data['customer_group_id'] = $this->getCustomerGroup($customerGroupId);
                    $data['subscription_status'] = $this->getSubscriptionStatus($customer->getId());
                    $data['account_created'] = $this->date->date('Y-m-d');
                    break;
            }

            $data['user_id'] = $customer->getId();
            $data['user_name'] = $customer->getName();
            $data['user_mail'] = $customer->getEmail();
            $data['timestamp'] = strtotime($this->date->date('Y-m-d'));
            $data['ip_address'] = Mage::helper('core/http')->getRemoteAddr();

            return $data;
        } catch (Exception $e) {
            $this->debug('Error message:' . $e->getMessage());

            return;
        }
    }

    /**
     * Get the customer subscription status
     *
     * @param $customerId
     * @return string
     */
    public function getSubscriptionStatus($customerId)
    {
        $status = '';

        try {
            $customerData = Mage::getModel('customer/customer')->load($customerId);
            $subscriber = Mage::getModel('newsletter/subscriber')->loadByCustomer($customerData);
            if ($subscriber->getSubscriberStatus() == self::STATUS_UNSUBSCRIBED) {
                $status = 'Unsubscribed';
            } elseif ($subscriber->getSubscriberStatus() == self::STATUS_SUBSCRIBED) {
                $status = 'Subscribed';
            } elseif ($subscriber->getSubscriberStatus() == self::STATUS_UNCONFIRMED) {
                $status = 'Unconfirmed';
            } elseif ($subscriber->getSubscriberStatus() == self::STATUS_NOT_ACTIVE) {
                $status = 'Not Activated';
            }
        } catch (Exception $e) {
            $this->debug('Error message:' . $e->getMessage());
        }

        return $status;
    }

    /**
     * Get the customer session info
     *
     * @param $customer
     * @return array
     */
    public function getCustomerSessionId($customer)
    {
        $data = array();
        try {
            $visitorData = Mage::getSingleton('core/session')->getVisitorData();
            $sessionId = $visitorData['session_id'] . strtotime(date('Y-m-d H:i:s'));
            $data['session_id'] = $sessionId;

            $trackingType = $this->getTrackingType();

            if ($trackingType == 1) {
                  // For persistent cart, no need to send previous_session_id. so we set as 0. previous_session_id mandatory field in TB.
                if(!$this->getPersistentCartCheck()){
                    $data['previous_session_id'] = $this->getFetchPointsByKey('_usid');
                }else{
                    $data['previous_session_id'] = '0'; 
                }
            } else {
                $data['previous_session_id'] = $this->getFetchPointsByKey('_usid');
                // $paramsData = base64_decode($this->cookie->get('tb_fetch_points'));
                // $paramsArray = explode('&', $paramsData);
                // if (count($paramsArray) > 0) {
                //     $trackingSession = explode('=', $paramsArray[1]);
                //     if (count($trackingSession) > 0) {
                //         $data['previous_session_id'] = $trackingSession[1];
                //     }
                // } else {
                //     $data['previous_session_id'] = $this->cookie->get('trackingsession');
                // }
            }

            /* To avoid user delete in TB when click from Abandoned cart mail - checkout button
            *  Already we are setting the action customer id.
            */
            $abandonedStatus  = Mage::getSingleton('checkout/session')->getAbandonedMerged();
            if($abandonedStatus){
                $data['previous_session_id'] = '0';
            }

            Mage::getSingleton('core/session')->setCustomerSessionId($sessionId);
        } catch (Exception $e) {
            $this->debug('Error message:' . $e->getMessage());
        }

        return $data;
    }

    /**
     * Check the order placed by registered user or not
     *
     * @param $order
     * @return boolean|Mage_Core_Model_Abstract
     */
    public function isRegisterCheckout($order)
    {
        $checkoutMethod = Mage::getModel('sales/quote')->load($order->getQuoteId())->getCheckoutMethod(true);
        if ($checkoutMethod !== 'register') {
            return false;
        }

        return Mage::getModel('customer/customer')->load($order->getCustomerId());
    }

    /**
     * Basic visit info
     *
     * @return array
     */
    public function visitInfo()
    {
        $data = array();
        try {
            $customer = Mage::getSingleton('customer/session');
            $data['user_name'] = $customer->isLoggedIn() ? $customer->getCustomer()->getName() : self::ANONYMOUS_USER;
            $data['user_email'] = $customer->isLoggedIn() ? $customer->getCustomer()->getEmail() : self::ANONYMOUS_USER;
            $trackingType = $this->getTrackingType();

            if ($trackingType == 1) {
                $data['already_tracked'] = true;
                $userId = $this->getFetchPointsByKey('_usid');
                $data['user_id'] = $customer->isLoggedIn() ? $customer->getCustomer()->getId() : $userId;
                $data['session_id'] = $customer->isLoggedIn() ? Mage::getSingleton('core/session')->getCustomerSessionId() : $userId;
            } else {
                $userId = Mage::getSingleton('core/session')->getTrackingSessionId();
                $data['user_id'] = $customer->isLoggedIn() ? $customer->getCustomer()->getId() : $userId;
                $data['session_id'] = $customer->isLoggedIn() ? Mage::getSingleton('core/session')->getCustomerSessionId() : $userId;
            }

            $data['page_url'] = Mage::helper('core/url')->getCurrentUrl();

            // Send these values when PHP tracking enabled.
            if ($trackingType == 0) {
                $data['ip_address'] = Mage::helper('core/http')->getRemoteAddr();
                $data['user_agent'] = Mage::helper('core/http')->getHttpUserAgent();
            }

            $pageTitle = Mage::app()->getLayout()->getBlock('head') ? Mage::app()->getLayout()->getBlock('head')->getTitle() : Mage::getSingleton('checkout/session')->getTitle();
            //$quotes = array("'", '"');
            //$replace = array('', '');

            $data['page_title'] = Mage::helper('core')->quoteEscape($pageTitle);

            if ($this->cookie->get('utm_source')) {
                $data['utm_sources'] = $this->cookie->get('utm_source');
            }

            if ($this->cookie->get('utm_token')) {
                $data['utm_token'] = $this->cookie->get('utm_token');
            }

            if ($this->cookie->get('utm_medium')) {
                $data['utm_medium'] = $this->cookie->get('utm_medium');
            }

            if ($this->cookie->get('utm_campaign')) {
                $data['utm_campaign'] = $this->cookie->get('utm_campaign');
            }
        } catch (Exception $e) {
            $this->debug('Error message:' . $e->getMessage());
        }

        return $data;
    }

    /**
     * Push the referer url
     *
     * @return boolean|array
     */
    public function getRefererData()
    {
        try {
            $domainName = $_SERVER['SERVER_NAME'];
            $payPal = 'paypal.com';
            $authorize = 'authorize.net';
            $referer = Mage::helper('core/http')->getHttpReferer();

            // base url and referer url matches.
            if ($referer === '' ||
                strpos($referer, $domainName) !== false ||
                strpos($referer, $payPal) !== false ||
                strpos($referer, $authorize) !== false) {
                return false;
            }

            $data = $this->visitInfo();
            $data['referrer_url'] = $referer;

            if (! in_array('', $data, true)
                || ! in_array(null, $data, true)
                || ! in_array(false, $data, true)) {
                return $data;
            }
        } catch (Exception $e) {
            $this->debug('Error message:' . $e->getMessage());

            return false;
        }
    }

    /**
     * Push the page info using Varien_Http_Client
     *
     * @param $url
     * @param $jsonData
     * @return string
     */
    public function postPageInfo($url, $jsonData)
    {
        $this->debug('EndpointUrl===>' . $url);
        $json = utf8_encode($jsonData);

        // ToDo: Authorization
        //$auth = $appID . ':' . base64_encode(hash_hmac('sha256', $json, $secretKey));

        $client = new Varien_Http_Client();
        $client->setUri($url);
        $client->setConfig(array(
            'maxredirects' => 0,
            'timeout' => 1
        ));
        //$client->setHeaders(array('Content-Length: ' . strlen($json), 'Authorization: ' . $auth));
        //$client->setRawData($json, 'application/json;charset=UTF-8');
        $client->setRawData($json);
        $this->debug('Request===>' . $jsonData);

        try {
            $response = $client->request(Varien_Http_Client::POST)->getBody();

            return $response;
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());

            return;
        }
    }

    /**
     * Remove the cookies
     */
    public function removeCookies()
    {
         Mage::getSingleton('core/cookie')->delete('trackingsession');
         Mage::getSingleton('core/cookie')->delete('trackingid');
        // $this->cookie->delete('trackingemail');
        // $this->cookie->delete('trackingname');
        // $this->cookie->delete('trackingorderid');
        // $this->cookie->delete('tb_data');
        // $this->cookie->delete('utm_source');
        // $this->cookie->delete('utm_token');
        // $this->cookie->delete('utm_medium');
        // $this->cookie->delete('utm_campaign');
            $this->cookie->delete('targetbay_session_id');
        // $this->cookie->delete('user_loggedin');
        // $this->cookie->delete('afterlogin_session_id');
        $this->cookie->delete('tb_fetch_points');
    }

    /**
     * Get the cart info
     *
     * @return array
     */
    public function getCartInfo()
    {
        $data = array();
        try {
            $customer = Mage::getSingleton('customer/session');
            $trackingType = $this->getTrackingType();
            if ($trackingType == 1) {
                $targetBaySessionId = $this->getFetchPointsByKey('_usid');
                $data['user_id'] = $customer->isLoggedIn() ? $customer->getCustomer()->getId() : $targetBaySessionId;

                $data['session_id'] = $customer->isLoggedIn() ? Mage::getSingleton('core/session')->getCustomerSessionId() : $targetBaySessionId;
            } else {
                $data['user_id'] = $customer->isLoggedIn() ? $customer->getId() : Mage::getSingleton('core/session')->getTrackingSessionId();
                $data['session_id'] = $customer->isLoggedIn() ? Mage::getSingleton('core/session')->getCustomerSessionId() : Mage::getSingleton('core/session')->getTrackingSessionId();
            }

            $data['user_name'] = $customer->isLoggedIn() ? $customer->getCustomer()->getName() : Targetbay_Tracking_Helper_Data::ANONYMOUS_USER;
            $data['user_mail'] = $customer->isLoggedIn() ? $customer->getCustomer()->getEmail() : Targetbay_Tracking_Helper_Data::ANONYMOUS_USER;

            $data['order_id'] = Mage::getSingleton('checkout/session')->getQuoteId();

            if ($this->cookie->get('utm_source')) {
                $data['utm_sources'] = $this->cookie->get('utm_source');
            }

            if ($this->cookie->get('utm_token')) {
                $data['utm_token'] = $this->cookie->get('utm_token');
            }

            if ($this->cookie->get('utm_medium')) {
                $data['utm_medium'] = $this->cookie->get('utm_medium');
            }

            if ($this->cookie->get('utm_campaign')) {
                $data['utm_campaign'] = $this->cookie->get('utm_campaign');
            }

            $data['timestamp'] = strtotime($this->date->date('Y-m-d'));
            $pageTitle = Mage::app()->getLayout()->getBlock('head') ? Mage::app()->getLayout()->getBlock('head')->getTitle() : Mage::getSingleton('checkout/session')->getTitle();
            $data['page_title'] = Mage::helper('core')->escapeHtml($pageTitle);
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $data;
    }

    /**
     * Get the common info for quote or order, billing and shipping
     *
     * @param $object
     * @return array
     */
    public function getSessionInfo($object)
    {
        $data = array();

        try {
            $trackingType = $this->getTrackingType();
            $customerId = $object->getCustomerId();
            if ($trackingType == 1) {
                $targetBaySessionId = $this->getFetchPointsByKey('_usid');
                $data['user_id'] = $customerId ? $customerId : $targetBaySessionId;
                $data['session_id'] = Mage::getSingleton('customer/session')->isLoggedIn() ? Mage::getSingleton('core/session')->getCustomerSessionId() : $targetBaySessionId;
            } else {
                $data['user_id'] = $customerId ? $customerId : Mage::getSingleton('core/session')->getTrackingSessionId();
                $data['session_id'] = Mage::getSingleton('customer/session')->isLoggedIn() ? Mage::getSingleton('core/session')->getCustomerSessionId() : Mage::getSingleton('core/session')->getTrackingSessionId();
            }

            if ($this->cookie->get('utm_source')) {
                $data['utm_sources'] = $this->cookie->get('utm_source');
            }

            if ($this->cookie->get('utm_token')) {
                $data['utm_token'] = $this->cookie->get('utm_token');
            }

            if ($this->cookie->get('utm_medium')) {
                $data['utm_medium'] = $this->cookie->get('utm_medium');
            }

            if ($this->cookie->get('utm_campaign')) {
                $data['utm_campaign'] = $this->cookie->get('utm_campaign');
            }

            $data['timestamp'] = strtotime($this->date->date('Y-m-d'));
            $pageTitle = Mage::app()->getLayout()->getBlock('head') ? Mage::app()->getLayout()->getBlock('head')->getTitle() : Mage::getSingleton('checkout/session')->getTitle();
            $data['page_title'] = Mage::helper('core')->escapeHtml($pageTitle);
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $data;
    }

    /**
     * Get the Order data
     *
     * @param $object
     * @return array
     */
    public function getInfo($object)
    {
        $data = array();

        try {
            $data = $this->getSessionInfo($object);
            $data['first_name'] = $object->getCustomerFirstname();
            $data['last_name'] = $object->getCustomerLastname();
            $guestUsername = $object->getCustomerFirstname() . ' ' . $object->getCustomerLastname();
            $gName = !empty($guestUsername) ? $guestUsername : self::ANONYMOUS_USER;
            $data['user_name'] = $object->getCustomerIsGuest() ? $gName : $data['first_name'] . ' ' . $data['last_name'];
            $data['user_mail'] = $object->getCustomerEmail();
            $data['order_id'] = $object->getId();
            $data['order_price'] = $object->getGrandTotal();
            $data['order_quantity'] = $object->getData('total_qty_ordered');
            $data['shipping_method'] = $object->getData('shipping_description');
            $data['shipping_price'] = $object->getData('shipping_amount');
            $data['tax_amount'] = $object->getData('tax_amount');
            $data['coupon_code'] = $object->getData('coupon_code');
            $data['discount_amount'] = $object->getData('discount_amount');
            $data['payment_method'] = $object->getPayment()->getMethodInstance()->getTitle();
            $data['cart_items'] = $this->getOrderItemsInfo($object);
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $data;
    }

    /**
     * Get the Order data
     *
     * @param $object
     * @return array
     */
    public function getOrderInfo($object)
    {
        $data = array();

        try {
            $data['user_id'] = $object->getCustomerIsGuest() ? self::ANONYMOUS_USER : $object->getCustomerId();
            $data['first_name'] = $object->getCustomerFirstname();
            $data['last_name'] = $object->getCustomerLastname();
            $guestUsername = $object->getCustomerFirstname() . ' ' . $object->getCustomerLastname();
            $gName = !empty($guestUsername) ? $guestUsername : self::ANONYMOUS_USER;
            $data['user_name'] = $object->getCustomerIsGuest() ? $gName : $data['first_name'] . ' ' . $data['last_name'];
            $data['user_mail'] = $object->getCustomerEmail();
            $data['order_id'] = $object->getId();
            $data['order_status'] = $object->getStatus();
            $data['order_price'] = $object->getSubtotal();
            $data['order_quantity'] = $object->getData('total_qty_ordered');
            $data['shipping_method'] = $object->getData('shipping_description');
            $data['shipping_price'] = $object->getData('shipping_amount');
            $data['tax_amount'] = $object->getData('tax_amount');
            $data['payment_method'] = $object->getPayment()->getMethodInstance()->getTitle();
            $data['cart_items'] = $this->getOrderItemsInfo($object);
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $data;
    }

    /**
     * Collect the order items info
     *
     * @param $order
     * @param $orderExportApi
     * @return array
     */
    public function getOrderItemsInfo($order, $orderExportApi = false)
    {
        $dataItems = array();
        try {
            $items = $order->getAllVisibleItems();
            foreach ($items as $orderItem) {
                $dataItem = $this->getItemInfo($orderItem);
                if ($customOptions = $this->getCustomOptionsInfo($orderItem, $orderExportApi)) {
                    $dataItem['attributes'] = $customOptions;
                }
                $dataItems[] = $dataItem;
            }
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $dataItems;
    }

    /**
     * Collect the order payment info.
     *
     * @param $orderId
     * @param $orderExportApi
     * @return string
     */
    public function getPaymentInfo($orderId, $orderExportApi = false)
    {
        $paymentTitle = '';
        $paymentMethod = '';

        try {
            $paymentCollection = Mage::getModel('sales/order_payment')
                ->getCollection()
                ->addFieldToFilter('parent_id', $orderId);
            foreach ($paymentCollection as $paymentDetails) {
                $paymentMethod = $paymentDetails->getMethod();
            }

            $store = Mage::app()->getStore();
            $path = 'payment/' . $paymentMethod . '/title';
            $paymentTitle = Mage::getStoreConfig($path, $store);
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $paymentTitle;
    }

    /**
     * Get the item info
     *
     * @param $item
     * @param $actionType
     * @return array
     */
    public function getItemInfo($item, $actionType = false, $productQty = null)
    {
        $dataItem = array();

        try {
            $product = Mage::getModel('catalog/product')->load($item->getData('product_id'));
            $dataItem['type'] = $item->getProductType();

            switch ($item->getProductType()) {
                case 'configurable':
                    $dataItem['parent_id'] = $item->getProductId();
                    if ($option = $item->getOptionByCode('simple_product')) {
                        $productId = $option->getProduct()->getId();
                        $dataItem['product_id'] = $productId;
                    } else {
                        $dataItem['product_id'] = "";
                        $simpleProduct = Mage::getModel('catalog/product')->loadByAttribute('sku', $item->getData('sku'));
                        if($simpleProduct){
                            $dataItem['product_id'] = $simpleProduct->getId();
                        }
                    }
                    break;
                case 'grouped':
                    if ($actionType !== 'add-to-cart') {
                        $productOption = $item->getProductOptions();
                        $productInfo = $productOption['info_buyRequest'];
                        if (isset($productInfo)) {
                            $dataItem['parent_id'] = $productInfo['super_product_config']['product_id'];
                        }
                        $dataItem['product_id'] = $item->getProductId();
                        break;
                    }
                    // ToDo: Need break?
                    // no break
                case 'bundle':
                    $itemIds = array();
                    $dataItem['parent_id'] = $item->getProductId();
                    $dataItem['product_id'] = $item->getProductId();
                    if ($actionType === 'add-to-cart') {
                        $session = Mage::getSingleton('checkout/session');
                        foreach ($session->getQuote()->getAllItems() as $items) {
                            if ($item->getId() == $items->getParentItemId()) {
                                $simpleProductId = $items->getProductId();
                                $simpleProductQty = $items->getQty();
                                $itemIds[$simpleProductId] = $simpleProductQty;
                            }
                        }
                        $dataItem['bundled_product'] = array_filter($itemIds);
                    } else {
                        $order = Mage::getModel('sales/order');
                        $order->load($item->getOrderId());

                        foreach ($order->getAllItems() as $items) {
                            if ($item->getId() == $items->getParentItemId()) {
                                $simpleProductId = $items->getProductId();
                                $simpleProductQty = $items->getQtyOrdered();
                                $itemIds[$simpleProductId] = ceil($simpleProductQty);
                            }
                        }
                        $dataItem['bundled_product'] = array_filter($itemIds);
                    }
                    break;
                default:
		    $dataItem['product_id'] = $item->getProductId();
		    $productParentId = Mage::getModel('tracking/tracking')->getFirstParentProductId($item->getProductId());
		    if($productParentId > 0){
			    	$dataItem['parent_id'] = $productParentId;
		    }
                    break;
            }

            $dataItem['product_sku'] = $item->getSku();
            $dataItem['product_name'] = Mage::helper('core')->escapeHtml($item->getName());
            $dataItem['price'] = $actionType ? $item->getData('price') : $item->getPrice();
            $dataItem['special_price'] = $product->getSpecialPrice();
            
            /* Mage::app()->getRequest()->getParam('qty') it's return null value in product list page addto cart.
               So now we take the product quantity from observer .
            */

            // $qty = $actionType ? Mage::app()->getRequest()->getParam('qty') : $item->getQty();
            $qty = $actionType ? $productQty : $item->getQty();

            if ($qty === null && $item->getQty()) {
                $qty = $item->getQty();
            }

            $dataItem['productimg'] = $this->getImageUrl($product, 'image');
            $dataItem['category'] = $this->getProductCategory($product);
            $dataItem['category_name'] = $this->getProductCategoryName($product);
            $dataItem['quantity'] = $item->getData('qty_ordered') ? $item->getData('qty_ordered') : $qty;
            $dataItem['page_url'] = Mage::getUrl($product->getUrlPath());
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $dataItem;
    }

    /**
     * Get the product categories
     *
     * @param $product
     * @return string
     */
    public function getProductCategoryName($product)
    {
        $categoryIds = $product->getCategoryIds();
        $productCategories = array();
        $categoryName = '';
        //$quotes = array("'", '"');
        //$replace = array('', '');

        if (count($categoryIds)) {
            foreach ($categoryIds as $categoryId) {
                $_category = Mage::getModel('catalog/category')->load($categoryId);
                $productCategories[] = Mage::helper('core')->escapeHtml($_category->getName());
            }

            $categoryName = implode(',', $productCategories);
        }

        return $categoryName;
    }

    /**
     * Get the product categories
     *
     * @param $product
     * @return string
     */
    public function getProductCategory($product)
    {
        $categoryIds = $product->getCategoryIds();
        $categoryId = '';
        if (count($categoryIds)) {
            $categoryId = implode(',', $categoryIds);
        }

        return $categoryId;
    }

    /**
     * Get the item custom options
     *
     * @param $item
     * @param $orderExportApi
     * @return array|bool
     */
    public function getCustomOptionsInfo($item, $orderExportApi)
    {
        $optionsData = array();
        try {
            $customOptions = $orderExportApi ? $item->getProductOptions() : $item->getProduct()->getTypeInstance(true)
                ->getOrderOptions($item->getProduct());

            if (empty($customOptions['options']) && empty($customOptions['attributes_info'])) {
                return false;
            }

            $superAttributeInfo = isset($customOptions['attributes_info']) ? $this->getOptionValues($customOptions['attributes_info']) : array();
            $customOptionInfo = isset($customOptions['options']) ? $this->getOptionValues($customOptions['options']) : array();
            $optionsData = array_merge($superAttributeInfo, $customOptionInfo);
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $optionsData;
    }

    /**
     * Get the options values
     *
     * @param $options
     * @return array
     */
    public function getOptionValues($options)
    {
        $optionData = array();
        try {
            foreach ($options as $option) {
                $data['label'] = $option['label'];
                $data['value'] = $option['value'];
                $optionData[] = $data;
            }
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $optionData;
    }

    /**
     * Get the order or quote address by address type.
     *
     * @param $object
     * @param $type
     * @return array
     */
    public function getAddressData($object, $type)
    {
        try {
            $address = ($type === self::SHIPPING) ? $object->getShippingAddress() : $object->getBillingAddress();
            $addressData = $this->getSessionInfo($object);
            $addressData['first_name'] = $address->getFirstname();
            $addressData['last_name'] = $address->getLastname();
            $guestUsername = $address->getFirstname() . ' ' . $address->getLastname();
            $gName = !empty($guestUsername) ? $guestUsername : self::ANONYMOUS_USER;
            $addressData['user_name'] = $object->getCustomerIsGuest() ? $gName : $addressData['first_name'] . ' ' . $addressData['last_name'];
            $addressData['order_id'] = $object->getId();
            $addressData['user_mail'] = $object->getCustomerEmail();
            $addressData['address1'] = $address->getStreet(1);
            $addressData['address2'] = $address->getStreet(2);
            $addressData['city'] = $address->getCity();
            $addressData['state'] = $address->getRegion();
            $addressData['zipcode'] = $address->getPostcode();

            $countryName = '';
            if ($address->getCountryId()) {
                $countryName = Mage::getModel('directory/country')->loadByCode($address->getCountryId())->getName();
            }

            $addressData['country'] = $countryName !== '' ? $countryName : $address->getCountryId();
            $addressData['phone'] = $address->getTelephone();

            $validationData = $this->getValidation($addressData);
            if ($validationData) {
                return $addressData;
            }
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
            return;
        }
    }

    /**
     * Get the product info
     *
     * @param $product
     * @return array
     */
    public function getProductData($product)
    {
        $configOptions = array();
        $customOptions = array();
        $data = array();
        try {
            $data['image_url'][] = $this->getProductImages($product);
            $data['entity_id'] = $product->getId();
            $data['attribute_set_id'] = $product->getEntityTypeId();
            $data['type_id'] = $product->getTypeId();
            $data['sku'] = $product->getSku();
            $data['product_status'] = $product->getStatus();
            $data['currency_type'] = Mage::app()->getStore()->getCurrentCurrencyCode();

            $stockItem = Mage::getModel('cataloginventory/stock_item');
            $stockItem->loadByProduct($product);
            $data['qty'] = $stockItem->getQty();
            $data['stock_count'] = $stockItem->getQty();
            $data['manage_stock'] = $stockItem->getManageStock();

            $data['visibility'] = $product->getVisibility();
            $data['description'] = $product->getDescription();
            $websiteId = Mage::app()->getRequest()->getParam('website_id');
            if ($websiteId) {
                $productPrice = Mage::getModel('catalog/product')
                    ->setStoreId($websiteId)
                    ->load($product->getId());
                $data['price'] = $productPrice->getPrice();
                $data['special_price'] = $productPrice->getSpecialPrice();
            } else {
                $data['price'] = $product->getPrice();
                $data['special_price'] = $product->getSpecialPrice();
            }
            $data['weight'] = $product->getWeight();
            $data['name'] = Mage::helper('core')->escapeHtml($product->getName());

            $data['category'] = $this->getProductCategory($product);
            if ($this->getGoogleProductReviewFeed()) {
                $upc = $product->getUpc();
                $upc1 = $product->getUpc1();

                if (isset($upc)) {
                    $data['upc'] = $upc;
                } elseif (isset($upc1)) {
                    $data['upc'] = $upc1;
                }

                if ($product->getBrand() && $product->getAttributeText('brand') !== '') {
                    $data['brand'] = $product->getAttributeText('brand');
                }

                if ($product->getMfgNo() !== '') {
                    $data['mpn'] = $product->getMfgNo();
                }
            }

            // Get product url key
            if ($product->getUrlKey() !== '') {
                $urlKey = $product->getUrlKey();
            } else {
                $urlKey = $product->getProductUrl();
            }

            $data['url_key'] = $urlKey;
            $data['full_url_key'] = $product->getProductUrl();

            if ($configData = $product->getData('configurable_attributes_data')) {
                $configOptions = $this->productOptions($configData, 'label');
            }

            if ($custOptions = $product->getData('product_options')) {
                $customOptions = $this->productOptions($custOptions);
            }

            $options = array_merge($configOptions, $customOptions);
            if (!empty($options)) {
                $data['attributes'] = $options;
            }
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $data;
    }

    /**
     * Get the product options when saving product
     *
     * @param $configData
     * @param string $customOption
     * @return array
     */
    public function productOptions($configData, $customOption = 'title')
    {
        $options = array();

        try {
            foreach ($configData as $cdata) {
                $attrLabel = $cdata[$customOption];
                if (!isset($cdata['values'])) {
                    $options[] = array(
                        'label' => $attrLabel,
                        'value' => $attrLabel
                    );
                    continue;
                }
                foreach ($cdata['values'] as $val) {
                    $attrVal = $val[$customOption];
                    $options[] = array(
                        'label' => $attrLabel,
                        'value' => $attrVal
                    );
                }
            }
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $options;
    }

    /**
     * Get the product images
     *
     * @param $product
     * @return array
     */
    public function getProductImages($product)
    {
        $images = array();

        try {
            $largeImage = $this->getImageUrl($product, 'image');
            $smallImage = $this->getImageUrl($product, 'small_image');
            $thumbImage = $this->getImageUrl($product, 'thumbnail');

            if (!empty($largeImage)) {
                $images['url'] = $largeImage;
            }

            if (!empty($smallImage)) {
                $images['thumbnail_image'] = $smallImage;
            }

            if (!empty($thumbImage)) {
                $images['medium_image'] = $thumbImage;
            }

            $images['position'] = 1;
            
            $multiImage = $product->getMediaGalleryImages();
            if (count($multiImage) > 0) {
                $otherImages =  array();
                foreach ($multiImage as $key => $_image) {
                    if ($largeImage != $_image->getUrl()) {
                    $otherImages[$key]['url'] = $_image->getUrl();
                    $otherImages[$key]['position'] = $_image->getPosition();
                    }
                }
                $images['other_images'] = $otherImages;
            } else {
                $images['other_images'] = NULL;
            }
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $images;
    }

    /**
     * Get the Product image url
     *
     * @param $product
     * @param $imageType
     * @return string
     */
    public function getImageUrl($product, $imageType)
    {
        $imgPath = '';
        if ($product->getData($imageType)) {
            $productImage = $product->getData($imageType);
            if (!empty($productImage) && $productImage !== 'no_selection') {
                $imgPath = Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_MEDIA) . 'catalog/product' . $product->getData($imageType);
            } else {
                $imgUrl = '';
                $productObj = Mage::getModel('catalog/product')->load($product->getId());
                foreach ($productObj->getMediaGalleryImages() as $pImage) {
                    if ($pImage->getFile() !== '') {
                        $imgUrl = Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_MEDIA) . 'catalog/product' . $pImage->getFile();
                    }
                }
                $imgPath = $imgUrl;
            }
        }
        if(!$imgPath){
            $imgPath = Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_SKIN) . 'frontend/base/default/images/catalog/product/placeholder/image.jpg';
        }

        return $imgPath;
    }

    /**
     * Check the order full fillment
     *
     * @return boolean
     */
    public function isFullFillmentProcess($params)
    {
        if (isset($params[self::ORDER_SHIPMENT]) ||
            isset($params[self::ORDER_INVOICE]) ||
            isset($params[self::ORDER_REFUND])
        ) {
            return true;
        }

        return false;
    }

    /**
     * Get the wishlist product info
     *
     * @param $productId
     * @return array
     */
    public function getWishlistProductInfo($productId)
    {
        $data = array();
        try {
            $product = Mage::getModel('catalog/product')->load($productId);
            $qty = 1;

            $data['type'] = $product->getTypeId();
            $data['product_id'] = $product->getId();
            $data['product_sku'] = $product->getSku();
            $data['name'] = Mage::helper('core')->escapeHtml($product->getName());
            $data['price'] = $product->getPrice();
            $data['special_price'] = $product->getSpecialPrice();
            $data['productimg'] = $this->getImageUrl($product, 'image');
            $data['category'] = $this->getProductCategory($product);
            $data['category_name'] = $this->getProductCategoryName($product);
            $data['quantity'] = Mage::app()->getRequest()->getParam('qty') ? Mage::app()->getRequest()->getParam('qty') : $qty;
            $data['page_url'] = Mage::getUrl($product->getUrlPath());
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $data;
    }

    /**
     * Collect the wishlist items info
     *
     * @param $wishlistInfo
     * @return array
     */
    public function getWishlistItemsInfo($wishlistInfo)
    {
        $dataItems = array();
        try {
            $wishlistItemCollection = $wishlistInfo->setStoreId(1)->getItemCollection();

            foreach ($wishlistItemCollection as $id => $wishlistItem) {
                $product = $wishlistItem->getProduct();
                $dataItem = $this->wishlistProductInfo($product);
                $dataItems[$id] = $dataItem;
            }
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $dataItems;
    }

    /**
     * Collect the wishlist items product info
     *
     * @param $product
     * @return array
     */
    public function wishlistProductInfo($product)
    {
        $dataItem = array();
        try {
            $product = Mage::getModel('catalog/product')->load($product->getData('entity_id'));
            $dataItem['type'] = $product->getTypeId();
            $dataItem['product_id'] = $product->getId();
            $dataItem['product_name'] = Mage::helper('core')->escapeHtml($product->getName());
            $dataItem['price'] = $product->getPrice();
            $dataItem['special_price'] = $product->getSpecialPrice();
            $dataItem['productimg'] = $this->getImageUrl($product, 'image');
            $dataItem['category'] = $this->getProductCategory($product);
            $dataItem['category_name'] = $this->getProductCategoryName($product);
            $dataItem['page_url'] = Mage::getUrl($product->getUrlPath());
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $dataItem;
    }

    /**
     *
     * @param $order
     * @param $params
     * @return array
     */
    public function getFullFillmentData($order, $params)
    {
        $shipmentItems = array();
        $shipmentsInfo = array();
        try {
            if (isset($params[self::ORDER_SHIPMENT])) {
                $shipmentsInfo['order_id'] = $order->getId();
                $shipmentsInfo['order_status'] = $order->getStatus();
                $shipmentsInfo['total_ordered_qty'] = (int) $order->getData('total_qty_ordered');
                $shipmentsInfo['user_id'] = $order->getData('customer_is_guest') ? self::ANONYMOUS_USER : $order->getData('customer_id');
                $shipmentsInfo['user_mail'] = $order->getData('customer_email');
                $shipmentsInfo['created_at'] = $order->getData('updated_at');

                foreach ($order->getAllVisibleItems() as $item) {
                    if ($item->getQtyShipped() === '') {
                        continue;
                    }

                    $shipmentItemInfo['product_id'] = $item->getProductId();
                    $shipmentItemInfo['name'] = Mage::helper('core')->escapeHtml($item->getName());
                    $shipmentItemInfo['sku'] = $item->getSku();
                    $shipmentItemInfo['qty_ordered'] = (int) $item->getQtyOrdered();
                    $shipmentItemInfo['qty_shipped'] = (int) $item->getQtyShipped();
                    $shipmentItems[] = $shipmentItemInfo;
                }
                $shipmentsInfo['shipment_items'] = $shipmentItems;
            }
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $shipmentsInfo;
    }

    /**
     * Stop the page visit for already tracked events.
     *
     * @param $eventName
     * @return bool
     */
    public function eventAlreadyTracked($eventName)
    {
        $stopAction = array(
            'cms_index_index',
            'catalog_category_view',
            'catalog_product_view',
            'catalogsearch_result_index',
            'catalogsearch_ajax_suggest',
            'checkout_cart_add',
            'checkout_cart_index',
            'checkout_cart_updatePost',
            'checkout_cart_remove',
            'customer_account_create',
            'customer_account_loginPost',
            'customer_account_logout',
            'customer_account_logoutSuccess',
            'customer_account_createPost',
            'checkout_onepage_index',
            'onestepcheckout_index_index',
            'firecheckout_index_index',
            'checkout_onepage_saveBilling',
            'checkout_onepage_saveShipping',
            'checkout_onepage_saveShippingMethod',
            'checkout_onepage_savePayment',
            'checkout_onepage_getAdditional',
            'checkout_onepage_progress',
            'checkout_onepage_saveOrder'
        );

        return in_array($eventName, $stopAction, true);
    }

    /**
     * debugging
     *
     * @param $mess
     */
    public function debug($mess, $logFile = null)
    {
        if ($this->logEnabled()) {
            Mage::log($mess, null, $logFile ? $logFile : $this->getLogFileName());
        }
    }

    /**
     * debugging
     *
     * @return string
     */
    public function getProductVersion()
    {
        return (string) Mage::getConfig()->getModuleConfig('Targetbay_Tracking')->version;
    }

    /**
     * Visiting page info
     *
     * @return array
     */
    public function getPageVisitData()
    {
        Mage::getModel('tracking/observer')->setCookieValues();
        $request = Mage::app()->getRequest();
        $utmSource = $request->getParam('utm_source');
        $utmToken = $request->getParam('utm_token');
        $utmMedium = $request->getParam('utm_medium');
        $utmCampaign = $request->getParam('utm_campaign');

        // Set Token Values
        if ($utmSource !== '' && $utmSource !== NULL) {
            $this->cookie->delete('utm_source');
            $this->cookie->set('utm_source', $utmSource, null, null, null, null, false);
        }

        if ($utmToken !== '' && $utmToken !== NULL) {
            $this->cookie->delete('utm_token');
            $this->cookie->set('utm_token', $utmToken, null, null, null, null, false);
        }

        if ($utmMedium !== '' && $utmMedium !== NULL) {
            $this->cookie->delete('utm_medium');
            $this->cookie->set('utm_medium', $utmMedium, null, null, null, null, false);
        }

        if ($utmCampaign !== '' && $utmCampaign !== NULL) {
            $this->cookie->delete('utm_campaign');
            $this->cookie->set('utm_campaign', $utmCampaign, null, null, null, null, false);
        }

        // Page Visit Tracking.
        return $this->visitInfo();
    }

    /**
     * Category view page
     *
     * @return array
     */
    public function getCategoryViewData()
    {
        $category = Mage::registry('current_category');
        $data = $this->visitInfo();
        $request = Mage::app()->getRequest();
        $utmSource = $request->getParam('utm_source');
        $utmToken = $request->getParam('utm_token');
        $utmMedium = $request->getParam('utm_medium');
        $utmCampaign = $request->getParam('utm_campaign');

        // Set Token Values
        if ($utmSource !== '' && $utmSource !== NULL) {
            $this->cookie->delete('utm_source');
            $this->cookie->set('utm_source', $utmSource, null, null, null, null, false);
        }

        if ($utmToken !== '' && $utmToken !== NULL) {
            $this->cookie->delete('utm_token');
            $this->cookie->set('utm_token', $utmToken, null, null, null, null, false);
        }

        if ($utmMedium !== '' && $utmMedium !== NULL) {
            $this->cookie->delete('utm_medium');
            $this->cookie->set('utm_medium', $utmMedium, null, null, null, null, false);
        }

        if ($utmCampaign !== '' && $utmCampaign !== NULL) {
            $this->cookie->delete('utm_campaign');
            $this->cookie->set('utm_campaign', $utmCampaign, null, null, null, null, false);
        }
        //$quotes = array("'", '"');
        //$replace = array('', '');
        $data['category_id'] = $category->getId();
        $data['category_url'] = $category->getUrl();
        $data['category_name'] = Mage::helper('core')->escapeHtml($category->getName());

        return $data;
    }

    /**
     * Product view page
     *
     * @return array
     */
    public function getProductViewData()
    {
        $data = array();

        // Get the base visit info.
        try {
            $data = $this->visitInfo();
            $request = Mage::app()->getRequest();
            $utmSource = $request->getParam('utm_source');
            $utmToken = $request->getParam('utm_token');
            $utmMedium = $request->getParam('utm_medium');
            $utmCampaign = $request->getParam('utm_campaign');

            // Set Token Values
            if ($utmSource !== '' && $utmSource !== NULL) {
                $this->cookie->delete('utm_source');
                $this->cookie->set('utm_source', $utmSource, null, null, null, null, false);
            }

            if ($utmToken !== '' && $utmToken !== NULL) {
                $this->cookie->delete('utm_token');
                $this->cookie->set('utm_token', $utmToken, null, null, null, null, false);
            }

            if ($utmMedium !== '' && $utmMedium !== NULL) {
                $this->cookie->delete('utm_medium');
                $this->cookie->set('utm_medium', $utmMedium, null, null, null, null, false);
            }

            if ($utmCampaign !== '' && $utmCampaign !== NULL) {
                $this->cookie->delete('utm_campaign');
                $this->cookie->set('utm_campaign', $utmCampaign, null, null, null, null, false);
            }

            $product = Mage::registry('product');
            $categoryIds = $product->getCategoryIds();
            //$quotes = array("'", '"');
            //$replace = array('', '');

            if (count($categoryIds)) {
                $firstCategoryId = $categoryIds[0];
                $_category = Mage::getModel('catalog/category')->load($firstCategoryId);
                $data['category'] = Mage::helper('core')->escapeHtml($_category->getName());
            }

            $data['product_id'] = $product->getId();
            $data['product_name'] = Mage::helper('core')->quoteEscape($product->getName());
            $data['price'] = $product->getPrice();
            $data['special_price'] = $product->getSpecialPrice();
            $data['productimg'] = $product->getImageUrl();
            $data['stock'] = self::OUT_OF_STOCK;
            $stock = $product->getStockItem();

            if ($stock->getIsInStock()) {
                $data['stock'] = self::IN_STOCK;
            }
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $data;
    }

    /**
     * Catalog search result page
     *
     * @return array
     */
    public function getCatalogSearchResultData()
    {
        $data = array();

        try {
            //$quotes = array("'", '"');
            //$replace = array('', '');
            $keyword = Mage::app()->getRequest()->getParam('q');

            if (empty($keyword)) {
                return false;
            }

            $data = $this->visitInfo();
            $request = Mage::app()->getRequest();
            $utmSource = $request->getParam('utm_source');
            $utmToken = $request->getParam('utm_token');
            $utmMedium = $request->getParam('utm_medium');
            $utmCampaign = $request->getParam('utm_campaign');

            // Set Token Values.
            if ($utmSource !== '' && $utmSource !== NULL) {
                $this->cookie->delete('utm_source');
                $this->cookie->set('utm_source', $utmSource, null, null, null, null, false);
            }

            if ($utmToken !== '' && $utmToken !== NULL) {
                $this->cookie->delete('utm_token');
                $this->cookie->set('utm_token', $utmToken, null, null, null, null, false);
            }

            if ($utmMedium !== '' && $utmMedium !== NULL) {
                $this->cookie->delete('utm_medium');
                $this->cookie->set('utm_medium', $utmMedium, null, null, null, null, false);
            }

            if ($utmCampaign !== '' && $utmCampaign !== NULL) {
                $this->cookie->delete('utm_campaign');
                $this->cookie->set('utm_campaign', $utmCampaign, null, null, null, null, false);
            }
            $data['keyword'] = Mage::helper('core')->escapeHtml($keyword);
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $data;
    }

    /**
     * Get the page info for static page
     *
     * @return array
     */
    public function getPageInfo()
    {
        $req = Mage::app()->getRequest();
        $controllerName = $req->getControllerName();
        $moduleName = $req->getModuleName();
        $websiteId = Mage::app()->getStore()->getWebsiteId();

        $data = array();
        if ($moduleName === 'cms') {
            $data = $this->getPageVisitData();
        } elseif ($controllerName === 'category') {
            $data = $this->getCategoryViewData();
        } elseif ($controllerName === 'product') {
            $data = $this->getProductViewData();
        } elseif ($moduleName === 'catalogsearch') {
            $data = $this->getCatalogSearchResultData();
        }

        $data['index_name'] = $this->getApiIndex($websiteId);

        return $data;
    }

    /**
     * Get the api url for static pages
     *
     * @return string
     */
    public function getApiUrl()
    {
        $req = Mage::app()->getRequest();
        $controllerName = $req->getControllerName();
        $moduleName = $req->getModuleName();
        //$websiteId = Mage::app()->getStore()->getWebsiteId();

        $type = '';
        if ($moduleName === 'cms') {
            $type = 'page-visit';
        } elseif ($controllerName === 'category') {
            $type = 'category-view';
        } elseif ($controllerName === 'product') {
            $type = 'product-view';
        } elseif ($moduleName === 'catalogsearch') {
            $type = 'searched';
        }

        //return $type !== '' ? $this->getHostname() . $type . '?api_token=' . $this->getApiToken($websiteId) : '';
        return $type;
    }

    /**
     * Get the quote items
     *
     * @param $items
     * @param $orderExportApi
     * @return array
     */
    public function getQuoteItems($items, $orderExportApi = false)
    {
        $dataItems = array();
        try {
            foreach ($items as $item) {
                $dataItem = $this->getItemInfo($item);
                if ($customOptions = $this->getCustomOptionsInfo($item, $orderExportApi)) {
                    $dataItem['attributes'] = $customOptions;
                }
                $dataItems[$item->getId()] = $dataItem;
            }
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $dataItems;
    }

    /**
     * Get the customer address data based on the action
     *
     * @param $addressId
     * @return array
     */
    public function getCustomerAddressData($addressId)
    {
        $data = array();
        try {
            $address = Mage::getModel('customer/address');
            $customer = Mage::getSingleton('customer/session')->getCustomer();
            $data['user_id'] = $customer->getId();
            $data['user_name'] = $customer->getName();
            $data['user_mail'] = $customer->getEmail();
            if ($addressId) {
                $existsAddress = $address->load($addressId);
                $data['address_id'] = $addressId;
                $data['old_address'] = $this->getExistingAddressData($existsAddress);
            }

            $data['new_address'] = $this->getNewAddressData();
            $data['timestamp'] = strtotime($this->date->date('Y-m-d'));
            $data['ip_address'] = Mage::helper('core/http')->getRemoteAddr();
            $data['user_agent'] = Mage::helper('core/http')->getHttpUserAgent();
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $data;
    }

    /**
     * Get the customer address data based on the action
     *
     * @return array
     */
    public function getNewAddressData()
    {
        $request = Mage::app()->getRequest();
        $addressData = array();

        try {
            $addressData['firstname'] = $request->getParam('firstname');
            $addressData['lastname'] = $request->getParam('lastname');
            $addressData['company'] = $request->getParam('company');
            $addressData['street'] = implode('', $request->getParam('street'));
            $addressData['city'] = $request->getParam('city');
            $addressData['region_id'] = $request->getParam('region_id');
            $addressData['region'] = $request->getParam('region');
            $addressData['postcode'] = $request->getParam('postcode');
            $addressData['country_id'] = $request->getParam('country_id');
            $addressData['telephone'] = $request->getParam('telephone');
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $addressData;
    }

    /**
     * Get the customer existing address data.
     *
     * @param $existsAddress
     *
     * @return array
     */
    public function getExistingAddressData($existsAddress)
    {
        $addressData = array();
        try {
            $addressData['firstname'] = $existsAddress->getFirstname();
            $addressData['lastname'] = $existsAddress->getLastname();
            $addressData['company'] = $existsAddress->getCompany();
            $addressData['street'] = $existsAddress->getStreet();
            $addressData['city'] = $existsAddress->getCity();
            $addressData['region_id'] = $existsAddress->getRegionId();
            $addressData['region'] = $existsAddress->getRegion();
            $addressData['postcode'] = $existsAddress->getPostcode();
            $addressData['country_id'] = $existsAddress->getCountryId();
            $addressData['telephone'] = $existsAddress->getTelephone();
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $addressData;
    }

    /**
     * Get the TargetBay review count and rating for product
     *
     * @return array
     */
    public function getRichSnippets()
    {
        if (!$this->getGoogleProductReviewSnippet()) {
            return false;
        }

        $productId = Mage::registry('product')->getId();
        $cacheResponseId = 'product_tbreview_response_' . $productId;
        $cacheTbCountId = 'product_tbreview_count_' . $productId;
        $cacheTbQaCountId = 'qa_tbreview_count_' . $productId;
        //$responseBody = '';
        //$responseData = array();
        //$type = Targetbay_Tracking_Helper_Data::RATINGS_STATS;
        $websiteId = Mage::app()->getStore()->getWebsiteId();
        $cacheTag = 'block_html';

        try {
            $cache = Mage::app()->getCache();
            if (false !== ($responseBody = $cache->load($cacheResponseId))) {
                $responseBody = unserialize($responseBody);
            }

            if (false !== ($cacheTbReviewCount = $cache->load($cacheTbCountId))) {
                $cacheTbReviewCount = unserialize($cacheTbReviewCount);
            }

            if (false !== ($cacheTbQaCount = $cache->load($cacheTbQaCountId))) {
                $cacheTbQaCount = unserialize($cacheTbQaCount);
            }

            if (!empty($responseBody) && !Mage::helper('core')->isModuleEnabled('Amasty_Fpc') && !Mage::helper('core')->isModuleEnabled('Lesti_Fpc') &&
                ($responseBody->tbReviewCount !== '') &&
                (($responseBody->tbReviewCount == $cacheTbReviewCount) &&
                    ($responseBody->tbQACount == $cacheTbQaCount))) {
                return $responseBody->tbContent;
            } else {
                $data['product_id'] = Mage::registry('product')->getId();
                $data['index_name'] = Mage::helper('tracking')->getApiIndex($websiteId);
                /*
                $data['index_name'] = Mage::helper('tracking')->getApiIndex($websiteId);
                $data['qaDisplay'] = Mage::helper('tracking')->getGoogleQuestionReviewSnippet();
                $jsonData = json_encode($data);
                $apiToken = '?api_token=' . Mage::helper('tracking')->getApiToken($websiteId);
                $feedUrl = Mage::helper('tracking')->getHostname() . $type . $apiToken;
                */

                $helper = Mage::helper('tracking');
                $data['qaDisplay'] = $helper->getGoogleQuestionReviewSnippet();
                $jsonData = json_encode($data);
                $feedUrl = $helper->getHost(Targetbay_Tracking_Helper_Data::RATINGS_STATS, Mage::app()->getStore()->getWebsiteId());
                $response = $this->postPageInfo($feedUrl, $jsonData);

                $responseBody = json_decode($response);
                if (!empty($responseBody)) {
                    /*
                    Mage::app()->getCache()->remove($cacheResponseId);
                    Mage::app()->getCache()->remove($cacheTbCountId);
                    Mage::app()->getCache()->remove($cacheTbQaCountId);
                    Mage::app()->getCache()->save(serialize($responseBody), $cacheResponseId, array($cacheTag));
                    Mage::app()->getCache()->save(serialize($responseBody->tbReviewCount), $cacheTbCountId);
                    Mage::app()->getCache()->save(serialize($responseBody->tbQACount), $cacheTbQaCountId);
                    $responseData = $responseBody->tbContent;
                    */

                    $cache->remove($cacheResponseId);
                    $cache->remove($cacheTbCountId);
                    $cache->remove($cacheTbQaCountId);
                    $cache->save(serialize($responseBody), $cacheResponseId, array($cacheTag));
                    $cache->save(serialize($responseBody->tbReviewCount), $cacheTbCountId);
                    $cache->save(serialize($responseBody->tbQACount), $cacheTbQaCountId);

                    return $responseBody->tbContent;
                }
            }
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());

            return false;
        }
    }

    /**
     * Get the TargetBay review count and dynamic ids
     *
     * @return string
     */
    public function getTargetbayReviewId()
    {
        $itemRefData = array();
        $itemRef = '';

        try {
            $trackingSnippet = $this->getRichSnippets();
            if ($trackingSnippet['reviews_count'] > 0) {
                foreach ($trackingSnippet['reviews'] as $key => $aggregateReviewDetails) {
                    $itemRefData[] = 'tb-review-' . $key;
                }
                $itemRef = implode(' ', $itemRefData);
            }
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
        }

        return $itemRef;
    }

    /**
     * Get the TargetBay review count and rating for product
     *
     * @return array|bool
     */
    public function getSiteReviewSnippets()
    {
        // if (!$this->getGoogleSiteReviewSnippetHome()) {
        //     return false;
        // }

        //$responseData = array();
        //$responseData = '';
        $cacheResponseId = 'site_tbreview_response';
        $cacheSiteTbCountId = 'site_tbreview_count';
        //$type = Targetbay_Tracking_Helper_Data::RATINGS_STATS;
        $websiteId = Mage::app()->getStore()->getWebsiteId();
        $cacheTag = 'block_html';

        try {
            if (false !== ($responseBody = Mage::app()->getCache()->load($cacheResponseId))) {
                $responseBody = unserialize($responseBody);
            }

            if (false !== ($cacheSiteTbReviewCount = Mage::app()->getCache()->load($cacheSiteTbCountId))) {
                $cacheSiteTbReviewCount = unserialize($cacheSiteTbReviewCount);
            }

            if (!empty($responseBody) &&
                ($responseBody->reviews_count !== '') &&
                ($responseBody->reviews_count == $cacheSiteTbReviewCount)) {
                $averageScore = $responseBody->reviews_average;
                $reviewsCount = $responseBody->reviews_count;
                $reviewsDetails = $responseBody->reviews;

                $responseData = array(
                    'average_score' => $averageScore,
                    'reviews_count' => $reviewsCount,
                    'reviews' => $reviewsDetails
                );

                return $responseData;
            } else {
                /*
                $data['index_name'] = Mage::helper('tracking')->getApiIndex($websiteId);
                $jsonData = json_encode($data);
                $apiToken = '?api_token=' . Mage::helper('tracking')->getApiToken($websiteId);
                $feedUrl = Mage::helper('tracking')->getHostname() . $type . $apiToken;
                */

                $data = array();
                $data['index_name'] = Mage::helper('tracking')->getApiIndex($websiteId);
                $helper = Mage::helper('tracking');
                $feedUrl = $helper->getHost(Targetbay_Tracking_Helper_Data::RATINGS_STATS, Mage::app()->getStore()->getWebsiteId());
                $jsonData = json_encode($data);
                $response = $this->postPageInfo($feedUrl, $jsonData);
                $responseBody = json_decode($response);
                if (!empty($responseBody) && ($responseBody->reviews_count !== '')) {
                    /*
                    Mage::app()->getCache()->remove($cacheResponseId);
                    Mage::app()->getCache()->remove($cacheSiteTbCountId);
                    Mage::app()->getCache()->save(serialize($responseBody->reviews), $cacheResponseId, array($cacheTag));
                    Mage::app()->getCache()->save(serialize($responseBody->reviews_count), $cacheSiteTbCountId);
                    */
                    $cache = Mage::app()->getCache();
                    $cache->remove($cacheResponseId);
                    $cache->remove($cacheSiteTbCountId);
                    $cache->save(serialize($responseBody->reviews), $cacheResponseId, array($cacheTag));
                    $cache->save(serialize($responseBody->reviews_count), $cacheSiteTbCountId);

                    $averageScore = $responseBody->reviews_average;
                    $reviewsCount = $responseBody->reviews_count;
                    $reviewsDetails = $responseBody->reviews;
                    $responseData = array(
                        'average_score' => $averageScore,
                        'reviews_count' => $reviewsCount,
                        'reviews' => $reviewsDetails
                    );

                    return $responseData;
                }
            }
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
            return;
        }
    }

    /**
     * @param $data
     * @return bool
     */
    public function checkDataEmpty($data)
    {
        try {
            $checkedData = array();

            if (count($data) < 1) {
                return false;
            }

            foreach ($data as $key => $value) {
                $value = trim($value);
                if (empty($value) || $value === null) {
                    $checkedData[$key] = $value;
                }
            }

            if (count($checkedData) > 1) {
                return false;
            }
        } catch (Exception $e) {
            $this->debug('Error message => Ln:' . $e->getLine() . ', File: '. $e->getFile() .', Msg: '. $e->getMessage());
            return false;
        }

        return true;
    }

    /**
     * @param $customerGroupId
     * @return mixed
     */
    public function getCustomerGroup($customerGroupId)
    {
        $groupName = Mage::getModel('customer/group')->load($customerGroupId)->getCustomerGroupCode();

        return $groupName;
    }

    /**
     * @param $data
     * @return bool
     */
    public function getValidation($data)
    {
        if ((!empty($data['user_id'])
            || $data['user_id'] !== null
            || $data['user_id'] !== '') ||
             (!empty($data['session_id'])
            || $data['session_id'] !== null
            || $data['session_id'] !== '') ||
             (!empty($data['user_email'])
            || $data['user_email'] !== null
            || $data['user_email'] !== '')
            ) {
            return true;
        }

        return false;
    }

    /**
     * To update Fetch Point Cookie
     * @param String fetchKey
     * @param String | Mixed fetchValue
     * @return Base64 String
     */
    public function updateFetchPoints($fetchKey, $fetchValue){
        $TbFetchArray = $this->getFetchPoints();
        $TbFetchArray[$fetchKey]= $fetchValue;
        $TbFetchOpitons = http_build_query($TbFetchArray);
        Mage::getModel('core/cookie')->set('tb_fetch_points', base64_encode($TbFetchOpitons), null, null, null, null, false);
        return;
    }

     /**
     * To get Fetch Point Cookie
     *
     * @return Array
     */
    public function getFetchPoints(){
        $TbFetchOpitons = base64_decode($this->cookie->get('tb_fetch_points'));
        parse_str($TbFetchOpitons, $TbFetchArray);
        return $TbFetchArray;
    }

     /**
     * To get Fetch Point Cookie by key
     *
     * @param String fetchKey
     * @return Array
     */
    public function getFetchPointsByKey($fetchKey){
        $TbFetchArray = $this->getFetchPoints();
        if(isset($TbFetchArray[$fetchKey])){
            return $TbFetchArray[$fetchKey];
        }
        return false;
    }

     /**
     * To get Parent product id by child product id
     *
     * @param Int child_id
     * @return String
     */
	public function getParentIdbyChild($childProduct){
		if(!$childProduct->getId()){
			return;
		}
		$pIdforReview = $childProduct->getId();
		if($childProduct->getTypeId() == "simple"){
			$parentIds = Mage::getModel('catalog/product_type_configurable')->getParentIdsByChild($childProduct->getId());
			if(!$parentIds)
				$parentIds = Mage::getModel('catalog/product_type_grouped')->getParentIdsByChild($childProduct->getId());
			if(isset($parentIds[0])){
				$pIdforReview = $parentIds[0];
		 	}
		}
		$placeholder = "<div class='targetbay_star_container' id='".$pIdforReview."'></div>";
		return $placeholder;
	}
}
