<?php
/*
 * @author Targetbay
 * @copyright 2016 Sathishkumar Mariappan <sathishkumar.m@innoppl.com>
 * @package Targetbay_Tracking
 */

class Targetbay_Tracking_Helper_Inventary extends Mage_Core_Helper_Abstract
{
    /**
     * Rendered the tracking review template if tracking enabled
     *
     * @see Mage_Core_Block_Template::_construct()
     */
    public function _construct()
    {
        $this->helper = Mage::helper('tracking');
    }

    /**
     * Get the current product
     * @return Mage_Catalog_Model_Product
     */
    public function getProduct()
    {
        return Mage::registry('current_product');
    }

    /**
     * Get the product inventory information
     * @return Mage_Catalog_Model_Product
     */
    public function getProductInfo()
    {
        $currentProduct = $this->getProduct();
        $currentProductId = $this->getProduct()->getId();
        $currentProductTypeId = $this->getProduct()->getTypeId();
        $data =  array();

        switch ($currentProductTypeId) {
            case Mage_Catalog_Model_Product_Type::TYPE_CONFIGURABLE:
                $childProductIds = array();
                $product = Mage::getModel('catalog/product')->load($currentProductId);
                $childProducts = Mage::getModel('catalog/product_type_configurable')
                                    ->getUsedProducts(null, $currentProduct);
                $i=0;
                foreach ($childProducts as $child) {
                    if ($child->getStockItem()->getIsInStock() == 0) {
                        $childProductIds[] = $child->getId();
                    }
                }
                foreach ($childProductIds as $key => $value) {
                    $data[$i]['child_product_id'] = $value;
                    $options = $this->getConfigurableProductOption($currentProductId, $value);
                    $data[$i]['options'] = $options;
                    $i++;
                }
                break;

            case Mage_Catalog_Model_Product_Type::TYPE_GROUPED:
                $childProductIds = array();
                $product = Mage::getModel('catalog/product')->load($currentProductId);
                $childProducts = $product->getTypeInstance(true)->getAssociatedProducts($product);
                $i=0;
                foreach ($childProducts as $child) {
                    if ($child->getStockItem()->getIsInStock() == 0) {
                        $childProductIds[] = $child->getId();
                    }
                }
                foreach ($childProductIds as $key => $value) {
                    $data[$i]['child_product_id'] = $value;
                    $childProductName = $this->getProductNameOption($value);
                    $data[$i]['options'] = $childProductName;
                    $i++;
                }
                break;

            case Mage_Catalog_Model_Product_Type::TYPE_BUNDLE:
                $childProductIds = array();
                $product = Mage::getModel('catalog/product')->load($currentProductId);
                $selectionCollection = $product->getTypeInstance(true)
                                ->getSelectionsCollection($product->getTypeInstance(true)->getOptionsIds($product), $product);

                $i=0;
                foreach ($selectionCollection as $child) {
                    if ($child->getStockItem()->getIsInStock() == 0) {
                        $childProductIds[] = $child->getId();
                    }
                }
                foreach ($childProductIds as $key => $value) {
                    $data[$i]['child_product_id'] = $value;
                    $childProductName = $this->getProductNameOption($value);
                    $data[$i]['options'] = $childProductName;
                    $i++;
                }
                break;

            case Mage_Catalog_Model_Product_Type::TYPE_SIMPLE:
                $data['child_product_id'] = '';
                $data['options'] = '';
                break;

            default:
                break;
        }

        return json_encode($data);
    }

    /**
     * @param $childProductId
     * @return array
     */
    public function getProductNameOption($childProductId)
    {
        $productData =  array();
        $quotes = array("'", '"');
        $replace = array('', '');
        $childProduct = Mage::getModel('catalog/product')->setStoreId(Mage::app()->getStore()->getId())
                                ->load($childProductId);
        $productName = $childProduct->getName();
        $productData['name'] = str_replace($quotes, $replace, $productName);

        return $productData;
    }

    /**
     * @param $parentProductId
     * @param $childProductId
     * @return array
     */
    public function getConfigurableProductOption($parentProductId, $childProductId)
    {
        $parentProduct = Mage::getModel('catalog/product')->setStoreId(Mage::app()->getStore()->getId())
                                ->load($parentProductId);
        $attributes = $parentProduct->getTypeInstance(true)->getConfigurableAttributesAsArray($parentProduct);
        $childProduct = Mage::getModel('catalog/product')->setStoreId(Mage::app()->getStore()->getId())
                                ->load($childProductId);
        $optionLabel = array();
        foreach ($attributes as $key => $attribute) {
            foreach ($attribute['values'] as $value) {
                $childValue = $childProduct->getData($attribute['attribute_code']);
                if ($value['value_index'] == $childValue) {
                    $optionLabel[] = $attribute['store_label'] . ':' . $value['store_label'];
                }
            }
        }

        return $optionLabel;
    }

    /**
     * @param $parentProductId
     * @param $childProductId
     * @return array
     */
    public function getConfigurableProductOptionIds($parentProductId, $childProductId)
    {
        $parentProduct = Mage::getModel('catalog/product')->setStoreId(Mage::app()->getStore()->getId())
                                ->load($parentProductId);
        $attributes = $parentProduct->getTypeInstance(true)->getConfigurableAttributesAsArray($parentProduct);
        $childProduct = Mage::getModel('catalog/product')->setStoreId(Mage::app()->getStore()->getId())
                                ->load($childProductId);
        $optionValue = array();
        foreach ($attributes as $key => $attribute) {
            foreach ($attribute['values'] as $value) {
                $childValue = $childProduct->getData($attribute['attribute_code']);
                if ($value['value_index'] == $childValue) {
                    $attributeId = $attribute['attribute_id'];
                    $optionValue[$attributeId] = $value['value_index'];
                }
            }
        }

        return $optionValue;
    }

    /**
     * @param $parentProductId
     * @param $childProductId
     * @return array
     */
    public function getBundledProductOptionIds($parentProductId, $childProductId)
    {
        $bundledProduct = Mage::getModel('catalog/product')->setStoreId(Mage::app()->getStore()->getId())
                                ->load($parentProductId);

        $selectionCollection = $bundledProduct->getTypeInstance(true)->getSelectionsCollection(
            $bundledProduct->getTypeInstance(true)->getOptionsIds($bundledProduct),
            $bundledProduct
        );

        $bundledItems = array();
        foreach ($selectionCollection as $option) {
            $bundledItems[$option->option_id] = $option->selection_id;
        }

        return $bundledItems;
    }
}
