<?php

/*
 * @author Targetbay
 * @copyright 2016 Sathishkumar Mariappan <sathishkumar.m@innoppl.com>
 * @package Targetbay_Tracking
 */

class Targetbay_Tracking_Model_Observer
{
    protected $apiToken;
    protected $indexName;
    protected $targetBayHost;
    protected $helper;

    /**
     * Get the credentials from store configurations
     */
    public function __construct()
    {
        $this->helper = Mage::helper('tracking');
        $websiteId = Mage::app()->getStore()->getWebsiteId();
        //$this->apiToken = '?api_token=' . $this->helper->getApiToken($websiteId);
        $this->indexName = $this->helper->getApiIndex($websiteId);
        //$this->targetBayHost = $this->helper->getHostname();
    }

    public function generateVisitorData()
    {
        $trackingType = $this->helper->getTrackingType();

        if ($trackingType == 1) {
            $targetbaySessionId = $this->helper->getFetchPointsByKey('_usid');
            $trackingSession = $targetbaySessionId;
        } else {
            $visitor_data = Mage::getSingleton('core/session')->getVisitorData();
            $trackingSession = $visitor_data['visitor_id'] . strtotime(date('Y-m-d H:i:s'));
            $sess = Mage::getSingleton('core/session')->getTrackingSessionId();
            if (!$sess) {
                Mage::getSingleton('core/session')->setTrackingSessionId($trackingSession);
            }
            $this->setCookieValuesWithEncode();
        }

        //$this->setCookieValues();
        //$this->setCookieValuesWithEncode();
    }

    /**
     * Set the cookie values with data encoding for user differentiate.
     */
    public function setCookieValuesWithEncode()
    {
        if (Mage::getSingleton('customer/session')->isLoggedIn()) {
            $customer = Mage::getSingleton('customer/session')->getCustomer();
            $customerName = $customer->getName();
            $customerId = $customer->getId();
            $customerEmail = $customer->getEmail();
            $customerSession = Mage::getSingleton('core/session')->getCustomerSessionId() ? Mage::getSingleton('core/session')->getCustomerSessionId() : $customerId;
            $quoteId = Mage::getSingleton('checkout/session')->getQuoteId() ? Mage::getSingleton('checkout/session')->getQuoteId() : '';
            $cookieData = '_uid=' . $customerId;
            $cookieData .= '&_un=' . $customerName;
            $cookieData .= '&_uem=' . $customerEmail;
            $cookieData .= '&_utid=' . $customerId;
            $cookieData .= '&_usid=' . $customerId;
            $cookieData .= '&_uoid=' . $quoteId;
            $cookieData .= '&_ulogin=1';
            $cookieData .= '&_uasid=' . $customerSession;
        } else {
            $customerName = Targetbay_Tracking_Helper_Data::ANONYMOUS_USER;
            $customerEmail = Targetbay_Tracking_Helper_Data::ANONYMOUS_USER;
            $customerId = Mage::getSingleton('core/session')->getTrackingSessionId();
            $quoteId = Mage::getSingleton('checkout/session')->getQuoteId() ? Mage::getSingleton('checkout/session')->getQuoteId() : '';
            $cookieData = '_uid=' . $customerId;
            $cookieData .= '&_utid=' . $customerId;
            $cookieData .= '&_usid=' . $customerId;
            $cookieData .= '&_un=' . $customerName;
            $cookieData .= '&_uem=' . $customerEmail;

            if ($quoteId) {
                $cookieData .= '&_uoid=' . $quoteId;
            }
        }

        if ($this->helper->cookie->get('tb_fetch_points') ||
            $this->helper->cookie->get('tb_fetch_points') == '') {
            $this->helper->cookie->delete('tb_fetch_points');
            // 31536000 secs as 1 year.
            $this->helper->cookie->set('tb_fetch_points', base64_encode($cookieData), time()+31536000, null, null, null, false);
        }
    }

    /**
     * Set the cookie values for user differentiate.
     */
    public function setCookieValues()
    {
        // for logged user
        if (Mage::getSingleton('customer/session')->isLoggedIn()) {
            $customer = Mage::getSingleton('customer/session')->getCustomer();
            $customerName = $customer->getName();
            $customerId = $customer->getId();
            $customerEmail = $customer->getEmail();

            if ($this->helper->cookie->get('trackingid')) {
                $this->helper->cookie->delete('trackingid');
                $this->helper->cookie->set('trackingid', $customerId, null, null, null, null, false);
            }
            if ($this->helper->cookie->get('trackingsession')) {
                $this->helper->cookie->delete('trackingsession');
                $this->helper->cookie->set('trackingsession', $customerId, null, null, null, null, false);
            }
            Mage::getModel('core/cookie')->set('user_loggedin', true, null, null, null, null, false);
            Mage::getModel('core/cookie')->set('afterlogin_session_id', Mage::getSingleton('core/session')->getCustomerSessionId(), null, null, null, null, false);
        } else {
            // For anonymous user
            $customerName = Targetbay_Tracking_Helper_Data::ANONYMOUS_USER;
            $customerEmail = Targetbay_Tracking_Helper_Data::ANONYMOUS_USER;
            $customerId = Mage::getSingleton('core/session')->getTrackingSessionId();

            if (!$this->helper->cookie->get('trackingid')) {
                $this->helper->cookie->set('trackingid', $customerId, null, null, null, null, false);
            }
            if (!$this->helper->cookie->get('trackingsession')) {
                $this->helper->cookie->set('trackingsession',   Mage::getSingleton('core/session')->getTrackingSessionId(), null, null, null, null, false);
            }
        }

        $this->helper->cookie->set('trackingemail', $customerEmail, null, null, null, null, false);
        $this->helper->cookie->set('trackingname', $customerName, null, null, null, null, false);

        $quoteId = Mage::getSingleton('checkout/session')->getQuoteId() ? Mage::getSingleton('checkout/session')->getQuoteId() : '';
        $this->helper->cookie->set('trackingorderid', $quoteId, null, null, null, null, false);
    }

    /**
     * Visiting page info
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void
     */
    public function pushPageVisitData(Varien_Event_Observer $observer)
    {
        $request = Mage::app()->getRequest();

        $currentUri = sprintf("%s/%s/%s", $request->getModuleName(), $request->getControllerName(), $request->getActionName());

        $triggers = array(
            'checkout/onepage/savePayment',
            'checkout/onepage/saveOrder',
            'checkout/onepage/success',
            'onestepcheckout/index/saveAddressOnestepcheckout',
            'onestepcheckout/index/save_shipping',
            'onestepcheckout/index/is_valid_email'
        );

        if (in_array($currentUri, $triggers, true)) {
            return false;
        }

        // Page referrer Tracking
        $this->pushReferralData();
        $utmSource = $request->getParam('utm_source');
        $utmToken = $request->getParam('utm_token');
        $utmMedium = $request->getParam('utm_medium');
        $utmCampaign = $request->getParam('utm_campaign');

        // Set Token Values
        if ($utmSource !== '' && $utmSource !== NULL) {
            $this->helper->cookie->delete('utm_source');
            $this->helper->cookie->set('utm_source', $utmSource, null, null, null, null, false);
        }

        if ($utmToken !== '' &&  $utmToken !== NULL) {
            $this->helper->cookie->delete('utm_token');
            $this->helper->cookie->set('utm_token', $utmToken, null, null, null, null, false);
        }

        if ($utmMedium !== '' && $utmMedium !== NULL) {
            $this->helper->cookie->delete('utm_medium');
            $this->helper->cookie->set('utm_medium', $utmMedium, null, null, null, null, false);
        }

        if ($utmCampaign !== '' && $utmCampaign !== NULL) {
            $this->helper->cookie->delete('utm_campaign');
            $this->helper->cookie->set('utm_campaign', $utmCampaign, null, null, null, null, false);
        }
    }

    /**
     * Visiting page info
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void
     */
    public function cmsPageVisit(Varien_Event_Observer $observer)
    {
        if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::PAGE_VISIT)) {
            return false;
        }

        $request = Mage::app()->getRequest();
        $utmSource = $request->getParam('utm_source');
        $utmToken = $request->getParam('utm_token');
        $utmMedium = $request->getParam('utm_medium');
        $utmCampaign = $request->getParam('utm_campaign');

        // Set Token Values
        if ($utmSource !== '' && $utmSource !== NULL) {
            $this->helper->cookie->delete('utm_source');
            $this->helper->cookie->set('utm_source', $utmSource, null, null, null, null, false);
        }

        if ($utmToken !== '' && $utmToken !== NULL) {
            $this->helper->cookie->delete('utm_token');
            $this->helper->cookie->set('utm_token', $utmToken, null, null, null, null, false);
        }

        if ($utmMedium !== '' && $utmMedium !== NULL) {
            $this->helper->cookie->delete('utm_medium');
            $this->helper->cookie->set('utm_medium', $utmMedium, null, null, null, null, false);
        }

        if ($utmCampaign !== '' && $utmCampaign !== NULL) {
            $this->helper->cookie->delete('utm_campaign');
            $this->helper->cookie->set('utm_campaign', $utmCampaign, null, null, null, null, false);
        }

        $request = Mage::app()->getRequest();
        $trackingType = $this->helper->getTrackingType();
        $moduleName = $request->getModuleName();

        // Page Visit Tracking
        $data = $this->helper->visitInfo();

        if ($trackingType == 1 && (
            $request->getControllerName() === 'product' ||
                $request->getControllerName() === 'category' ||
                $moduleName === 'cms'
            )
        ) {
            return false;
        }

        $validationData = $this->helper->getValidation($data);
        if ($validationData) {
            $this->pushPages($data, Targetbay_Tracking_Helper_Data::PAGE_VISIT);
            return;
        }
    }

    /**
     * Add to cart
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void
     */
    public function pushAddToCart(Varien_Event_Observer $observer)
    {
        if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::ADDTOCART)) {
            return false;
        }

        Mage::getSingleton('checkout/session')->setTitle('Shopping cart');

        $productInfo = $observer->getProduct();

        $productEventInfo = $observer->getEvent()->getProduct();

        $quote = Mage::getModel('checkout/cart')->getQuote();
        $item = $quote->getItemByProduct($productEventInfo);
        $productQty = $observer->getEvent()->getProduct()->getQty();
        if ($productInfo->getId()) {
            try {
                switch ($productInfo->getData('type_id')) {
                    case 'grouped':
                        $productIds = Mage::app()->getRequest()->getParam('super_group');

                        $dataItem = array();
                        $productData = array();
                        foreach ($productIds as $id => $qty) {
                            if ($qty < 1) {
                                continue;
                            }
                            $product = Mage::getModel('catalog/product')->load($id);
                            $productData['type'] = $product->getTypeId();
                            $productData['parent_id'] = Mage::app()->getRequest()->getParam('product');
                            $productData['product_id'] = $id;
                            $productData['product_sku'] = $product->getSku();
                            $productData['product_name'] = Mage::helper('core')->escapeHtml($product->getName());
                            $productData['price'] = $product->getFinalPrice() * $qty;
                            $productData['special_price'] = $product->getSpecialPrice();
                            $productData['productimg'] = $this->helper->getImageUrl($product, 'image');
                            $productData['category'] = $this->helper->getProductCategory($product);
                            $productData['category_name'] = $this->helper->getProductCategoryName($product);
                            $productData['quantity'] = $qty;
                            $productData['page_url'] = Mage::getUrl($product->getUrlPath());
                            $productData['attributes'] = '';
                            $dataItem[] = $productData;
                        }

                        $data = $this->helper->getCartInfo();
                        $data['product_type'] = $productInfo->getData('type_id');
                        $data['cart_item'] = $dataItem;
                        break;
                    default:
                        $data = array_merge($this->helper->getCartInfo(), $this->helper->getItemInfo($item, Targetbay_Tracking_Helper_Data::ADDTOCART,$productQty));
                        $data['product_type'] = $item->getProductType();
                        $data['price'] = $item->getPrice();

                        if ($customOptions = $this->helper->getCustomOptionsInfo($item, null)) {
                            $data['attributes'] = $customOptions;
                        }
                        break;
                }

                $this->pushPages($data, Targetbay_Tracking_Helper_Data::ADDTOCART);
                if (!Mage::getSingleton('customer/session')->isLoggedIn()) {
                    Mage::getModel('tracking/log')->saveCartData();
                }
            } catch (Exception $e) {
                $this->helper->debug('ERROR: ' . $e->getMessage());
            }
        }
        return;
    }

    /**
     * Capture the Update cart event
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void
     */
    public function pushUpdateCartData(Varien_Event_Observer $observer)
    {
        if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::UPDATECART)) {
            return false;
        }
        $items = $observer->getEvent()->getCart()->getQuote()->getAllVisibleItems();
        $requestInfo = $observer->getEvent()->getInfo();
        $data = $this->helper->getCartInfo();

        foreach ($items as $item) {
            $newQty = $requestInfo[$item->getId()]['qty'];
            $oldQty = $item->getQty();
            if ($newQty == 0 || ($newQty == $oldQty)) {
                continue;
            }
            $itemData = $this->helper->getItemInfo($item);

            unset($itemData['quantity']);
            $newitemData = $itemData;
            $newitemData['old_quantity'] = $oldQty;
            $newitemData['new_quantity'] = $newQty;
            $data['cart_items'][] = $newitemData;
        }

        if (isset($data['cart_items'])) {
            $this->pushPages($data, Targetbay_Tracking_Helper_Data::UPDATECART);
        }

        if (!Mage::getSingleton('customer/session')->isLoggedIn()) {
                    Mage::getModel('tracking/log')->saveCartData();
        }

        return;
    }

     /**
     * Capture the Update / Edit cart for configurable product event
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void
     */
    public function pushEditCartData(Varien_Event_Observer $observer)
    {
        if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::UPDATECART)) {
            return false;
        }
        try{
            $updateItem = $observer->getItem();
            $request = $observer->getrequest();
            $qty = $request->getParam('qty');

            if($updateItem->getId() != $request->getParam('id')){
                $childItem = Mage::getModel("sales/quote_item")
                                ->getCollection()
                                ->setQuote($updateItem->getQuote())
                                ->addFieldToFilter("parent_item_id", $updateItem->getId())
                                ->getFirstItem();

                $data = array_merge($this->helper->getCartInfo(), $this->helper->getItemInfo($updateItem, Targetbay_Tracking_Helper_Data::ADDTOCART,$qty));
                $data['product_type'] = Mage::getModel('catalog/product')->load($updateItem->getProductId())->getData('type_id');
                $data['price'] = $updateItem->getPrice();

                if ($customOptions = $this->helper->getCustomOptionsInfo($updateItem, null)) {
                    $data['attributes'] = $customOptions;
                }
                $this->pushPages($data, Targetbay_Tracking_Helper_Data::ADDTOCART);
            }else{
                $oldQty = Mage::getSingleton('checkout/session')->getConfigOldQty();
                $newQty = $updateItem->getQty();

                $data = $this->helper->getCartInfo();

                $itemData = $this->helper->getItemInfo($updateItem);

                unset($itemData['quantity']);
                $newitemData = $itemData;
                $newitemData['old_quantity'] = $oldQty;
                $newitemData['new_quantity'] = $newQty;
                $data['cart_items'][] = $newitemData;
                $this->pushPages($data, Targetbay_Tracking_Helper_Data::UPDATECART);
            }

            if (!Mage::getSingleton('customer/session')->isLoggedIn()) {
                        Mage::getModel('tracking/log')->saveCartData();
            }

        } catch (Exception $e) {
                $this->helper->debug('ERROR: ' . $e->getMessage());
        }
        return;
    }

    /**
     * Capture the get configurable product quantity
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void
     */
    public function getConfigProductQty(Varien_Event_Observer $observer)
    {
        try{
            $requestInfo = Mage::app()->getRequest();
            $id = $requestInfo->getParam('id');
            $quoteItem = Mage::getSingleton('checkout/cart')->getQuote()->getItemById($id);
            if($quoteItem && $quoteItem->getId()){
                $qty = $quoteItem->getQty();
                Mage::getSingleton('checkout/session')->unsConfigOldQty();
                Mage::getSingleton('checkout/session')->setConfigOldQty($qty);
            }
        } catch (Exception $e) {
                $this->helper->debug('ERROR: ' . $e->getMessage());
        }
    }

    /**
     * Capture the Update cart for configurable product event
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void
     */
    public function pushUpdateCartConfigProduct(Varien_Event_Observer $observer)
    {
        if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::UPDATECART)) {
            return false;
        }

        $requestInfo = Mage::app()->getRequest();
        $id = $requestInfo->getParam('id');
        $newQty = $requestInfo->getParam('qty');
        $oldQty = Mage::getSingleton('checkout/session')->getConfigOldQty();

        if ($newQty == 0 || ($newQty == $oldQty)) {
            return false;
        }

        /*  Mage::log($productEventInfo->getId(), null,'mylogfile.log', true);*/

        $data = $this->helper->getCartInfo();
        $quoteItem = Mage::getSingleton('checkout/cart')->getQuote()->getItemById($id);
    
        /*
        echo "<pre>"; print_r( $stock->getData())    ; echo "</pre>";
        echo "<pre>"; echo "<pre>"; echo (int)$stock->getQty();echo "</pre>";
        echo "<pre>"; echo $stock->getMinQty();echo "</pre>";
        echo $requestInfo->getParam('qty');echo "</pre>";
        */

        $itemData = $this->helper->getItemInfo($quoteItem);

        unset($itemData['quantity']);
        $newitemData = $itemData;
        $newitemData['old_quantity'] = $oldQty;
        $newitemData['new_quantity'] = $newQty;
        $data['cart_items'][] = $newitemData;

        // print_r($newitemData); exit;

        // To fix the product quantity is not available in edit product page & update cart.
        // so we get the user typed quantity from $requestInfo->getParam('qty');
        // we get total quantity available from product modal. we compare both quantiy and checked.

        /*  $pId = $quoteItem->getProductId();
        $_product = Mage::getModel('catalog/product')->load($newitemData['product_id']);
        $stock = Mage::getModel('cataloginventory/stock_item')->loadByProduct($_product);

        if($newQty > (int) $stock->getQty()){
            return false;
        }*/

        if (isset($data['cart_items'])) {
            $this->pushPages($data, Targetbay_Tracking_Helper_Data::UPDATECART);
        }

        return;
    }

    /**
     * Push the empty cart actions data
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void|boolean
     */
    public function pushEmptyCartData(Varien_Event_Observer $observer)
    {
        $updateAction = (string) Mage::app()->getRequest()->getParam('update_cart_action');
        if ($updateAction != 'empty_cart') {
            return false;
        }
        $items = Mage::getSingleton('checkout/session')->getQuote()->getAllVisibleItems();
        foreach ($items as $item) {
            $this->removeCartItem($item);
        }

        /* Remove the cart tracking data in targetbay_tracking_log while empty the cart */
        if (!Mage::getSingleton('customer/session')->isLoggedIn()) {
            Mage::getModel('tracking/log')->removeCartData();
        }
        return;
    }

    /**
     * Push the remove cart item data
     *
     * @param Varien_Event_Observer $observer
     */
    public function pushRemoveCartItemData(Varien_Event_Observer $observer)
    {
        $this->removeCartItem($observer->getEvent()->getQuoteItem());
    }

    /**
     * Remove the cart item
     *
     * @param unknown $item
     *
     * @return void|boolean
     */
    public function removeCartItem($item)
    {
        if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::REMOVECART)) {
            return false;
        }
        
        $data = array_merge($this->helper->getCartInfo(), $this->helper->getItemInfo($item));
        if ($customOptions = $this->helper->getCustomOptionsInfo($item, null)) {
                $data['attributes'] = $customOptions;
            }
        $this->pushPages($data, Targetbay_Tracking_Helper_Data::REMOVECART);

        /*if (!Mage::getSingleton('customer/session')->isLoggedIn()) {
            Mage::getModel('tracking/log')->removeCartData();
        }*/

        if (!Mage::getSingleton('customer/session')->isLoggedIn()) {
                    Mage::getModel('tracking/log')->saveCartData();
        }

        return;
    }

    /**
     * Observe the checkout page
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void|boolean
     */
    public function pushCheckoutPageData(Varien_Event_Observer $observer)
    {
        if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::CHECKOUT)) {
            return false;
        }

        $request = Mage::app()->getRequest();
        $utmSource = $request->getParam('utm_source');
        $utmToken = $request->getParam('utm_token');
        $utmMedium = $request->getParam('utm_medium');
        $utmCampaign = $request->getParam('utm_campaign');

        // Set Token Values
        if ($utmSource !== '' && $utmSource !== NULL) {
            $this->helper->cookie->delete('utm_source');
            $this->helper->cookie->set('utm_source', $utmSource, null, null, null, null, false);
        }

        if ($utmToken !== '' && $utmToken !== NULL) {
            $this->helper->cookie->delete('utm_token');
            $this->helper->cookie->set('utm_token', $utmToken, null, null, null, null, false);
        }

        if ($utmMedium !== '' && $utmMedium !== NULL) {
            $this->helper->cookie->delete('utm_medium');
            $this->helper->cookie->set('utm_medium', $utmMedium, null, null, null, null, false);
        }

        if ($utmCampaign !== '' && $utmCampaign !== NULL) {
            $this->helper->cookie->delete('utm_campaign');
            $this->helper->cookie->set('utm_campaign', $utmCampaign, null, null, null, null, false);
        }

        $quote = Mage::getSingleton('checkout/session')->getQuote();

        $data = $this->helper->getCartInfo();
        $data['cart_items'] = $this->helper->getOrderItemsInfo($quote);
        $data['total_qty'] = Mage::getModel('checkout/cart')->getQuote()->getItemsQty();
        $data['total_amount'] = Mage::getModel('checkout/cart')->getQuote()->getGrandTotal();
		$data['page_url'] = Mage::helper('core/url')->getCurrentUrl();
        $this->pushPages($data, Targetbay_Tracking_Helper_Data::CHECKOUT);

        return;
    }

    /**
     * Order data
     *
     * @param Varien_Event_Observer $observer
     * @return Targetbay_Tracking_Model_Observer
     *
     * @return void|boolean
     */
    public function pushOrderData(Varien_Event_Observer $observer)
    {
        $isRegister = false;
        try {
            if (!Mage::registry('order_pushed')) {
                Mage::register('order_pushed', true);
                if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::ORDER_ITEMS)) {
                    return false;
                }
                $order = $observer->getEvent()->getOrder();
                $params = Mage::app()->getFrontController()->getRequest()->getParams();

                // Capture the customer registration.
                if ($customer = $this->helper->isRegisterCheckout($order)) {
                    $this->pushRegisterData($customer);
                    $isRegister = true;
                }

                // Order Data Push to the Tag Manager
                $orderInfo = $this->helper->getInfo($order);
                if ($isRegister) {
                    $orderInfo['registercheckout'] = true;
                }
                $this->pushPages($orderInfo, Targetbay_Tracking_Helper_Data::ORDER_ITEMS);
                Mage::getSingleton('checkout/session')->unsQuoteMerged();

                if (!Mage::getSingleton('customer/session')->isLoggedIn()) {
                    Mage::getModel('tracking/log')->removeCartData();
                }
            }
        } catch (Exception $e) {
            $this->helper->debug('ERROR: ' . $e->getMessage());
        }

        return;
    }

    /**
     * Order status change data
     *
     * @param Varien_Event_Observer $observer
     * @return Targetbay_Tracking_Model_Observer
     *
     * @return void|boolean
     */
    public function pushOrderStatusData(Varien_Event_Observer $observer)
    {
        try {
            $order = $observer->getEvent()->getOrder();
            $this->helper->setStoreIdForAdminEvents($order->getStoreId());
        	if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::ORDER_STATUS)) {
                return false;
            }

            $data = $this->helper->getSessionInfo($order);
            $data['first_name'] = $order->getCustomerFirstname();
            $data['last_name'] = $order->getCustomerLastname();
            $guestUsername = $order->getCustomerFirstname() . ' ' . $order->getCustomerLastname();
            $gName = !empty($guestUsername) ? $guestUsername : self::ANONYMOUS_USER;
            $data['user_name'] = $order->getCustomerIsGuest() ? $gName : $data['first_name'] . ' ' . $data['last_name'];
            $data['user_mail'] = $order->getCustomerEmail();
            $data['order_id'] = $order->getId();
            $data['status'] = $order->getStatus();
            if ($data['status'] === null) {
                return false;
            }
            $this->pushPages($data, Targetbay_Tracking_Helper_Data::ORDER_STATUS);
        } catch (Exception $e) {
            $this->helper->debug('ERROR: ' . $e->getMessage());
        }

        return;
    }

    /**
     * Abandoned cart merge for logged in user
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void
     */
    public function quoteMergeData(Varien_Event_Observer $observer)
    {
        $abandonedMail = Mage::getSingleton('core/session')->getAbandonedMail();
        if ($abandonedMail) {
            $CustomerQuote = Mage::getModel('sales/quote')
                ->setStoreId(Mage::app()->getStore()->getId())
                ->loadByCustomer(Mage::getSingleton('customer/session')->getCustomerId());

            $abandonedQuote =  $observer->getSource();
            try {
                foreach ($abandonedQuote->getAllVisibleItems() as $abandoneditem) {
                    foreach ($CustomerQuote->getAllItems() as $item) {
                        if ($item->getProductId() == $abandoneditem->getProductId()) {
                            $item->isDeleted(true);
                            if ($item->getHasChildren()) {
                                foreach ($item->getChildren() as $child) {
                                    $child->isDeleted(true);
                                }
                            }
                        }
                    }
                }
                $CustomerQuote->collectTotals()->save();
            } catch (\Exception $e) {
                $this->helper->debug('Error:' . $e->getMessage());
            }
        }
        return $this;
    }

    /**
     * Push the shipment data
     *
     * @param unknown $order
     * @param unknown $params
     * @return boolean
     */
    public function pushShipmentData($order, $params)
    {
        try {
            if ($this->helper->isFullFillmentProcess($params)) {
                $data = $this->helper->getFullFillmentData($order, $params);
                $this->pushPages($data, Targetbay_Tracking_Helper_Data::ORDER_SHIPMENT);
                return true;
            }
        } catch (Exception $e) {
            $this->helper->debug('ERROR: ' . $e->getMessage());
        }

        return false;
    }

    /**
     * Push the order shipment data
     *
     * @param Varien_Event_Observer $observer
     * @return Targetbay_Tracking_Model_Observer
     * @return boolean
     */
    public function pushOrderShipmentData(Varien_Event_Observer $observer)
    {
        try {

            $shipment = $observer->getEvent()->getShipment();
            $order = $shipment->getOrder();
            $this->helper->setStoreIdForAdminEvents($order->getStoreId());

        	if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::ORDER_SHIPMENT)) {
                return false;
            }

            $params[Targetbay_Tracking_Helper_Data::ORDER_SHIPMENT] = true;
            if ($this->helper->isFullFillmentProcess($params)) {
                $data = $this->helper->getFullFillmentData($order, $params);
                $this->pushPages($data, Targetbay_Tracking_Helper_Data::ORDER_SHIPMENT);
            }
        } catch (Exception $e) {
            $this->helper->debug('ERROR: ' . $e->getMessage());
        }
        return;
    }

    /**
     * Push billing and shipping info when user enter into onestepcheckout
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void|boolean
     */
    public function pushCheckoutAddressData(Varien_Event_Observer $observer)
    {
        if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::ONESTEPCHECKOUT_ADDRESS)) {
            return false;
        }
        Mage::getSingleton('checkout/session')->setTitle('OneStepCheckout');

        $quote = Mage::getSingleton('checkout/session')->getQuote();
        $billingInfo = $this->helper->getAddressData($quote, Targetbay_Tracking_Helper_Data::BILLING);

        if (! in_array('', $billingInfo, true)
            || ! in_array(null, $billingInfo, true)
            || ! in_array(false, $billingInfo, true)) {
            $this->pushPages($billingInfo, Targetbay_Tracking_Helper_Data::BILLING);
        }

        $shippingInfo = $this->helper->getAddressData($quote, Targetbay_Tracking_Helper_Data::SHIPPING);

        if (! in_array('', $shippingInfo, true)
            || ! in_array(null, $shippingInfo, true)
            || ! in_array(false, $shippingInfo, true)) {
            $this->pushPages($shippingInfo, Targetbay_Tracking_Helper_Data::SHIPPING);
        }

        return;
    }

    /**
     * Push the data when customer loggedin
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void|boolean
     */
    public function customerLoginPushData(Varien_Event_Observer $observer)
    {
        if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::LOGIN)) {
            return false;
        }
        if (!$observer->getCustomer()) {
            return false;
        }
        $this->setCookieValuesWithEncode();
        $data = $this->helper->getCustomerData($observer->getCustomer(), Targetbay_Tracking_Helper_Data::LOGIN);
        $this->pushPages($data, Targetbay_Tracking_Helper_Data::LOGIN);

        return;
    }

    /**
     * Push the data when customer logout
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void|boolean
     */
    public function customerLogoutPushData(Varien_Event_Observer $observer)
    {
        if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::LOGOUT)) {
            return false;
        }
        $data = $this->helper->getCustomerData($observer->getCustomer(), Targetbay_Tracking_Helper_Data::LOGOUT);
        $this->pushPages($data, Targetbay_Tracking_Helper_Data::LOGOUT);

        
        // if persistant cart enabled, no need to delete cookie while logout.
        if(!$this->helper->getPersistentCartCheck()){
            // Remove all Cookies
            $this->helper->removeCookies();
        }

        // Remove all Cookies
        // $this->helper->removeCookies();

        return;
    }

    /**
     * Registration observer
     *
     * @param Varien_Event_Observer $observer
     */
    public function customerRegisterPushData(Varien_Event_Observer $observer)
    {
        $this->pushRegisterData($observer->getCustomer());
    }

    /**
     * Push the registration data
     *
     * @param unknown $customer
     *
     * @return void|boolean
     */
    public function pushRegisterData($customer)
    {
        if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::CREATE_ACCOUNT)) {
            return false;
        }
        $data = $this->helper->getCustomerData($customer, Targetbay_Tracking_Helper_Data::CREATE_ACCOUNT);
        $this->pushPages($data, Targetbay_Tracking_Helper_Data::CREATE_ACCOUNT);

        return;
    }

    /**
     * Push the newsletter subscription data
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void|boolean
     */
    public function pushSubscriptionData(Varien_Event_Observer $observer)
    {
        if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::SUBSCRIBE_CUSTOMER)) {
            return false;
        }

        $websiteId = Mage::app()->getStore()->getWebsiteId();
        $customerModel = Mage::getModel('customer/customer');

        if (Mage::app()->getRequest()->getParam('email')) {
            $email = Mage::app()->getRequest()->getParam('email');
            $customerData = $customerModel->setWebsiteId($websiteId)->loadByEmail($email);
            $customerId = $customerData->getEntityId();
        } else {
            $customerId = Mage::getSingleton('customer/session')->getCustomer()->getId();
            $customerData = $customerModel->load($customerId);
            $email = '';
        }

        $data = $this->helper->visitInfo();

        if (empty($email)) {
            $subscriberFactory = Mage::getModel('newsletter/subscriber')->loadByCustomer($customerData);

            if ($subscriberFactory->getSubscriberStatus() == Targetbay_Tracking_Helper_Data::STATUS_UNSUBSCRIBED) {
                $status = 'Unsubscribed';
            } elseif ($subscriberFactory->getSubscriberStatus() == Targetbay_Tracking_Helper_Data::STATUS_SUBSCRIBED) {
                $status = 'Subscribed';
            } elseif ($subscriberFactory->getSubscriberStatus() == Targetbay_Tracking_Helper_Data::STATUS_UNCONFIRMED) {
                $status = 'Unconfirmed';
            } elseif ($subscriberFactory->getSubscriberStatus() == Targetbay_Tracking_Helper_Data::STATUS_NOT_ACTIVE) {
                $status = 'Not Activated';
            } else {
                $status = Mage::app()->getRequest()->getParam('status');
            }
        } else {
            $status = '';
        }

        $status = !empty($email) ? 'Subscribed' : $status;
        if (Mage::getSingleton('customer/session')->isLoggedIn()) {
            $email = Mage::getSingleton('customer/session')->getCustomer()->getEmail();
            $data['user_mail'] = $email;
        } else {
            $data['user_mail'] = $email;
        }
        $data['subscription_status'] = $status;

        if (!empty($status) && !empty($email)) {
            $this->pushPages($data, Targetbay_Tracking_Helper_Data::SUBSCRIBE_CUSTOMER);
        }
        return;
    }

    /**
     * Admin Account Activation observer
     *
     * @param Varien_Event_Observer $observer
     */
    public function customerAdminConfirmData(Varien_Event_Observer $observer)
    {
        $customerInfo = $observer->getCustomer()->getData();
        $this->helper->setStoreIdForAdminEvents($customerInfo['store_id']);
        $data = array_merge($this->helper->visitInfo(), $customerInfo);
        $this->pushPages($data, Targetbay_Tracking_Helper_Data::ADMIN_ACTIVATE_ACCOUNT);

        return;
    }

    /**
     * Admin Account Delete observer
     *
     * @param Varien_Event_Observer $observer
     */
    public function deleteCustomerData(Varien_Event_Observer $observer)
    {
        $customerIds = Mage::app()->getRequest()->getParam('customer');
        $customerModel = Mage::getModel('customer/customer');
        if (count($customerIds) > 0) {
            foreach ($customerIds as $key => $customerId) {
                if ($customerId !== '') {
                    $customer = $customerModel->load($customerId);
                    $data['user_id'] = $customerId;
                    $data['user_mail'] = $customer->getEmail();
                    $data['website_id'] = $customer->getWebsiteId();
                    $data['store_id'] = $customer->getStoreId();
                    $data['group_id'] = $customer->getGroupId();
                    $data['date'] = $this->helper->date->date('Y-m-d');
                    $data['timestamp'] = strtotime($this->helper->date->date('Y-m-d'));
                    $this->helper->setStoreIdForAdminEvents($customer->getStoreId());
                    $this->pushPages($data, Targetbay_Tracking_Helper_Data::ADMIN_DELETE_ACCOUNT);
                }
            }
        } else {
            $customer = Mage::registry('current_customer');
            if (isset($customer) && $customer->getId()) {
                $customer->load($customer->getId());
                $storeId = $customer->getStoreId();
                $data['user_id'] = $customer->getId();
                $data['user_mail'] = $customer->getEmail();
                $data['website_id'] = $customer->getWebsiteId();
                $data['store_id'] = $customer->getStoreId();
                $data['group_id'] = $customer->getGroupId();
                $data['date'] = $this->helper->date->date('Y-m-d');
                $data['timestamp'] = strtotime($this->helper->date->date('Y-m-d'));
                $this->helper->setStoreIdForAdminEvents($customer->getStoreId());
                $this->pushPages($data, Targetbay_Tracking_Helper_Data::ADMIN_DELETE_ACCOUNT, $storeId);
            }
        }
        return;
    }

    /**
     * Add and update product
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void
     */
    public function pushProductData(Varien_Event_Observer $observer)
    {
        try {
            $product = $observer->getEvent()->getProduct();
            $websiteId = $product->getWebsiteIds();
            $storeId = $product->getStoreId();
            $this->helper->setStoreIdForAdminEvents($storeId);

        	if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::ADD_PRODUCT)) {
                return false;
            }
            $param = Mage::app()->getRequest()->getParams();

            if ($product->getId()) {
                $type = Targetbay_Tracking_Helper_Data::ADD_PRODUCT;
                if (isset($param['id'])) {
                    if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::UPDATE_PRODUCT)) {
                        return false;
                    }
                    $type = Targetbay_Tracking_Helper_Data::UPDATE_PRODUCT;
                }
                $data = $this->helper->getProductData($product);
                $this->pushPages($data, $type, $storeId);
            }
        } catch (Exception $e) {
            $this->helper->debug('ERROR: ' . $e->getMessage());
        }

        return;
    }

    /**
     * Delete product
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void
     */
    public function pushDeleteProductData(Varien_Event_Observer $observer)
    {
        try {
            $product = $observer->getEvent()->getProduct();
            $storeId = $product->getStoreId();
            $this->helper->setStoreIdForAdminEvents($storeId);

        	if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::DELETE_PRODUCT)) {
                return false;
            }
            $params = Mage::app()->getRequest()->getParams();

            if ($params) {
                $data['entity_id'] = Mage::app()->getRequest()->getParam('id');
                $data['user_name'] = Targetbay_Tracking_Helper_Data::ANONYMOUS_USER;
                if ($this->helper->getFetchPointsByKey('_usid')) {
                    $data['user_id'] = $this->helper->getFetchPointsByKey('_usid');
                    $data['session_id'] = $this->helper->getFetchPointsByKey('_usid');
                } else {
                    $data['user_id'] = strtotime(date('Y-m-d H:i:s'));
                    $data['session_id'] = strtotime(date('Y-m-d H:i:s'));
                }
                $data['date'] = $this->helper->date->date('Y-m-d');
                $data['timestamp'] = strtotime($this->helper->date->date('Y-m-d'));
                $data['time'] = $this->helper->date->date('H:i');
                $data['user_mail'] = Targetbay_Tracking_Helper_Data::ANONYMOUS_USER;
                $this->pushPages($data, Targetbay_Tracking_Helper_Data::DELETE_PRODUCT);
            }
        } catch (Exception $e) {
            $this->helper->debug('ERROR: ' . $e->getMessage());
        }

        return;
    }

    /**
     * Category view page
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void
     */
    public function pushCategoryViewData(Varien_Event_Observer $observer)
    {
        if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::CATEGORY_VIEW)) {
            return false;
        }
        try {
            $category = Mage::registry('current_category');
            $data = $this->helper->visitInfo();
            if (!empty($category)) {
                $data['category_id'] = $category->getId();
                $data['category_url'] = $category->getUrl();
                $data['category_name'] = $category->getName();
            }

            $trackingType = $this->helper->getTrackingType();
            if ($trackingType != 1) {
                $this->pushPages($data, Targetbay_Tracking_Helper_Data::CATEGORY_VIEW);
            }

            return;
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
            return;
        }
    }

    /**
     * Product view page
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void
     */
    public function pushProductViewData(Varien_Event_Observer $observer)
    {
        if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::PRODUCT_VIEW)) {
            return false;
        }
        try {
            // Get the base visit info.
            $data = $this->helper->visitInfo();
            $product = Mage::registry('product');
            if (!empty($product)) {
                $categoryIds = $product->getCategoryIds();

                if (count($categoryIds)) {
                    $firstCategoryId = $categoryIds[0];
                    $_category = Mage::getModel('catalog/category')->load($firstCategoryId);
                    $data['category'] = $_category->getName();
                }

                $data['product_id'] = $product->getId();
                $data['product_name'] = $product->getName();
                $data['price'] = $product->getPrice();
                $data['productimg'] = $product->getImageUrl();
                $data['stock'] = Targetbay_Tracking_Helper_Data::OUT_OF_STOCK;
                $stock = $product->getStockItem();

                if ($stock->getIsInStock()) {
                    $data['stock'] = Targetbay_Tracking_Helper_Data::IN_STOCK;
                }

				$request = Mage::app()->getRequest();
				$utmSource = $request->getParam('utm_source');
				$utmToken = $request->getParam('utm_token');
				$utmMedium = $request->getParam('utm_medium');
				$utmCampaign = $request->getParam('utm_campaign');

				// Set Token Values
				if ($utmSource !== '' && $utmSource !== NULL) {
				    $data['utm_source'] = $utmSource;
				}

				if ($utmToken !== '' && $utmToken !== NULL) {
				    $data['utm_token'] = $utmToken;
				}

				if ($utmMedium !== '' && $utmMedium !== NULL) {
				    $data['utm_medium'] = $utmMedium;
				}

				if ($utmCampaign !== '' && $utmCampaign !== NULL) {
				    $data['utm_campaign'] = $utmCampaign;
				}

                $trackingType = $this->helper->getTrackingType();
                $validationData = $this->helper->getValidation($data);
                if ($trackingType != 1 && (! in_array('', $data, true)
                        || ! in_array(null, $data, true)
                        || ! in_array(false, $data, true))) {
                    $this->pushPages($data, Targetbay_Tracking_Helper_Data::PRODUCT_VIEW);
                }
            }
            return;
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
            return;
        }
    }

     /**
     * Product status mass update.
       In magento admin panel, option to change the status.
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void
     */
    public function massProductStatusUpdate(Varien_Event_Observer $observer)
    {
        try{

            $param = Mage::app()->getRequest()->getParams();
            // $product = $observer->getEvent()->getProduct();
            // $pIds = $observer->getEvent()->getData();

            // get the total product id is in array.
            $productIds = array();
            if(isset($param['product']) && count($param['product']) > 0 ){
                $data['productIds'] = $param['product'];
                $data['status'] = $param['status'];
                $this->pushPages($data,Targetbay_Tracking_Helper_Data::PRODUCT_STATUS_MASSUPDATE);
            }
            
        } catch (Exception $e) {
            $this->helper->debug('ERROR: ' . $e->getMessage());
        }

        return;
    }

    /*
        Product attributes change by import product csv.
        @param Varien_Event_Observer $observer
        @return void
    */

    public function importProductUpdate(Varien_Event_Observer $observer)
    {

        try{
            // get all product ids from user uploaded csv 
            $adapter = $observer->getEvent()->getAdapter();
            $productIds = $adapter->getAffectedEntityIds(); 

            if(count($productIds) > 0) {
                $data['productIds'] = $productIds;
                // Mage::log(print_r($data),1), null,'mylogfile.log', true);
                $this->pushPages($data,Targetbay_Tracking_Helper_Data::PRODUCT_UPDATE_IMPORT);
            }
        }catch (Exception $e) {
            $this->helper->debug('ERROR: ' . $e->getMessage());
        }

        return;
    }


    /**
     * Push the searhced query string
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void
     */
    public function pushSearchQueryData(Varien_Event_Observer $observer)
    {
        if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::CATALOG_SEARCH)) {
            return false;
        }

        $quotes = array("'", '"');
        $replace = array('', '');
        $keyword = Mage::app()->getRequest()->getParam('q');

        if (empty($keyword)) {
            return false;
        }

        $data = $this->helper->visitInfo();
        $data['keyword'] = Mage::helper('core')->escapeHtml($keyword);
        $trackingType = $this->helper->getTrackingType();

        if ($trackingType != 1) {
            $this->pushPages($data, Targetbay_Tracking_Helper_Data::CATALOG_SEARCH);
        }

        return;
    }

    /**
     * Add the Wishlist
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void
     */
    public function pushAddWishlistData($observer)
    {
        if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::WISHLIST)) {
            return false;
        }
        $wishlistItems = $observer->getEvent()->getItems();

        foreach ($wishlistItems as $item) {
            if ($item->getParentItem()) {
                $item = $item->getParentItem();
            }

            $itemInfo = $this->helper->getWishlistProductInfo($item->getData('product_id'));

            $data = array_merge($this->helper->visitInfo(), $itemInfo);
            $data['item_id'] = $item->getWishlistItemId();

            if ($customOptions = $this->helper->getCustomOptionsInfo($item, null)) {
                $data['attributes'] = $customOptions;
            }
            $this->pushPages($data, Targetbay_Tracking_Helper_Data::WISHLIST);
        }

        return;
    }

    /**
     * Update the Wishlist items
     *
     * @param Varien_Event_Observer $observer
     */
    public function pushUpdateWishlistData(Varien_Event_Observer $observer)
    {
        if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::UPDATE_WISHLIST)) {
            return false;
        }

        $wishlistId = Mage::app()->getRequest()->getParam('wishlist_id');
        $wishlistDesc = Mage::app()->getRequest()->getParam('description');
        $wishlistQty = Mage::app()->getRequest()->getParam('qty');

        $data = $this->helper->visitInfo();
        $items = array();
        $data['wishlist_id'] = $wishlistId;

        foreach ($wishlistDesc as $id => $item) {
            $wishlistItem = Mage::getModel('wishlist/item')->load($id);
            $items[$id]['item_id'] = $id;
            $items[$id]['product_id'] = $wishlistItem->getProductId();
            $items[$id]['store_id'] = $wishlistItem->getStoreId();
            $items[$id]['description'] = $item;
            $items[$id]['qty'] = $wishlistQty[$id];
        }

        $data['wishlist_items'] = $items;
        $this->pushPages($data, Targetbay_Tracking_Helper_Data::UPDATE_WISHLIST);

        return;
    }

    /**
     * Remove the Wishlist items
     *
     * @param Varien_Event_Observer $observer
     */
    public function removeWhislistItem(Varien_Event_Observer $observer)
    {
        if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::REMOVE_WISHLIST)) {
            return false;
        }

        $id = (int) Mage::app()->getRequest()->getParam('item');
        $item = Mage::getModel('wishlist/item')->load($id);

        if (!$item->getId()) {
            return false;
        }

        $wishlist = Mage::getModel('wishlist/wishlist')->load($item->getWishlistId());

        if (!$wishlist) {
            return false;
        } else {
            $data = $this->helper->visitInfo();
            $data['item_id'] = $id;
            $data['product_id'] = $item->getProductId();
            $data['store_id'] = $item->getStoreId();
            $data['wishlist_id'] = $item->getWishlistId();
            $this->pushPages($data, Targetbay_Tracking_Helper_Data::REMOVE_WISHLIST);
        }

        return;
    }

    /**
     * Visiting page info
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void
     */
    public function pushCartData(Varien_Event_Observer $observer)
    {
        if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::CART_INDEX)) {
            return false;
        }

        $request = Mage::app()->getRequest();
        $utmSource = $request->getParam('utm_source');
        $utmToken = $request->getParam('utm_token');
        $utmMedium = $request->getParam('utm_medium');
        $utmCampaign = $request->getParam('utm_campaign');

        // Set Token Values
        if ($utmSource !== '' && $utmSource !== NULL) {
            $this->helper->cookie->delete('utm_source');
            $this->helper->cookie->set('utm_source', $utmSource, null, null, null, null, false);
        }

        if ($utmToken !== '' && $utmToken !== NULL) {
            $this->helper->cookie->delete('utm_token');
            $this->helper->cookie->set('utm_token', $utmToken, null, null, null, null, false);
        }

        if ($utmMedium !== '' && $utmMedium !== NULL) {
            $this->helper->cookie->delete('utm_medium');
            $this->helper->cookie->set('utm_medium', $utmMedium, null, null, null, null, false);
        }

        if ($utmCampaign !== '' && $utmCampaign !== NULL) {
            $this->helper->cookie->delete('utm_campaign');
            $this->helper->cookie->set('utm_campaign', $utmCampaign, null, null, null, null, false);
        }

        // Page Visit Tracking
        $data = $this->helper->visitInfo();
        $this->pushPages($data, Targetbay_Tracking_Helper_Data::CART_INDEX);

        return;
    }

    /**
     * Push customer address data
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void
     */
    public function pushCustomerAddressData(Varien_Event_Observer $observer)
    {
        if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::CUSTOMER_ADDRESS)) {
            return false;
        }

        $addressId = Mage::app()->getRequest()->getParam('id');
        $data = $this->helper->getCustomerAddressData($addressId);
        $this->pushPages($data, Targetbay_Tracking_Helper_Data::CUSTOMER_ADDRESS);

        return;
    }

    /**
     * Push customer account data
     *
     * @param Varien_Event_Observer $observer
     *
     * @return void
     */
    public function pushCustomerAccountData(Varien_Event_Observer $observer)
    {
        if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::CUSTOMER_ACCOUNT)) {
            return false;
        }

        Mage::getSingleton('customer/session')->setTitle('Account Information');
        $data = $this->helper->visitInfo();
        $data['customer_id'] = Mage::getSingleton('customer/session')->getCustomer()->getId();
        $data['firstname'] = Mage::getSingleton('customer/session')->getCustomer()->getFirstname();
        $data['lastname'] = Mage::getSingleton('customer/session')->getCustomer()->getLastname();
        $data['email'] = Mage::getSingleton('customer/session')->getCustomer()->getEmail();
        $data['account_updated'] = Mage::getModel('core/date')->date('Y-m-d');
        $this->pushPages($data, Targetbay_Tracking_Helper_Data::CUSTOMER_ACCOUNT);

        return;
    }

    /**
     * Push the referrer data.
     *
     * @return boolean
     */
    public function pushReferralData()
    {
        if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::PAGE_REFERRAL)) {
            return false;
        }

        try {
            $request = Mage::app()->getRequest();
            $trackingType = $this->helper->getTrackingType();
            $moduleName = $request->getModuleName();

            if ($trackingType == 1 && (
                $request->getControllerName() === 'product'
                    || $request->getControllerName() === 'category'
                    || $moduleName === 'cms'
            )
            ) {
                return false;
            }

            if ($referrerData = $this->helper->getRefererData()) {
                $this->pushPages($referrerData, Targetbay_Tracking_Helper_Data::PAGE_REFERRAL);
            }
        } catch (Exception $e) {
            $this->debug('Error message ' . $e->getMessage());
            return;
        }
    }

    /**
     * API Calls
     *
     * @param $data
     * @param $type
     * @param array $websiteId
     */
    public function pushPages($data, $type, $storeId = null)
    {
        try {
            if ($storeId != '') {
                // foreach ($websiteId as $webId) {
                    //$apiToken = $this->helper->getApiToken($webId);
                    $indexName = $this->helper->getApiIndex($storeId);
                    if($indexName){
                        $data['index_name'] = $indexName;
                        //$endPoint = $this->targetBayHost . $type . '?api_token=' . $apiToken;
                        //$this->helper->postPageInfo($endPoint, json_encode($data));
                        $this->helper->postPageInfo($this->helper->getHost($type, $storeId), json_encode($data));
                    }
                // }
            } else {
                //$apiToken = $this->apiToken;
                $indexName = $this->indexName;
                $data['index_name'] = $indexName;
                //$endPoint = $this->targetBayHost . $type . $apiToken;
                //$this->helper->postPageInfo($endPoint, json_encode($data));
                $this->helper->postPageInfo($this->helper->getHost($type), json_encode($data));
            }
        } catch (Exception $e) {
            $this->helper->debug($type . 'ERROR:' . $e->getMessage());
        }
    }

    /**
     * add Product To Cart using Coupon
     *
     * @return boolean
     */
    public function addProductToCart()
    {
        $productSku = Mage::helper('tracking')->getProductSkuToRedeem();

        if (!$productSku) {
            return;
        }

        $productSkuCollection = explode(',', $productSku);
        $quote = Mage::getSingleton('checkout/session')->getQuote();
        $storeId = Mage::app()->getStore()->getId();
        $productIds =  array();
        $qty = 1;

        try {
            if (Mage::app()->getRequest()->getParam('remove') == 1) {
                $cartHelper = Mage::helper('checkout/cart');
                $items = $cartHelper->getCart()->getItems();
                if (count($productSkuCollection) > 1) {
                    foreach ($productSkuCollection as $sku) {
                        $product = Mage::getModel('catalog/product')->setStoreId($storeId)->loadByAttribute('sku', $sku);
                        $productIds[] = $product->getId();
                    }
                } else {
                    $product = Mage::getModel('catalog/product')->setStoreId($storeId)->loadByAttribute('sku', $productSku);
                    $productIds[] = $product->getId();
                }
                foreach ($items as $item) {
                    $itemId = $item->getItemId();
                    if (in_array($item->getProduct()->getId(), $productIds, true)) {
                        $cartHelper->getCart()->removeItem($itemId)->save();
                    }
                }
            } else {
                $couponCode = Mage::app()->getRequest()->getParam('coupon_code');
                $conditionsSku = '';

                if ($couponCode !== '') {
                    $oCoupon = Mage::getModel('salesrule/coupon')->load($couponCode, 'code');
                    $oRule = Mage::getModel('salesrule/rule')->load($oCoupon->getRuleId());
                    $actionRule = unserialize($oRule->getActionsSerialized());

                    if (isset($actionRule) && $actionRule['conditions']) {
                        $conditions = $actionRule['conditions'];
                        if (count($conditions) === 1) {
                            $conditionsSku = explode(',', $conditions[0]['value']);
                        }
                    }

                    try {
                        if (count($productSkuCollection) > 1) {
                            foreach ($productSkuCollection as $sku) {
                                $product = Mage::getModel('catalog/product')->setStoreId($storeId)->loadByAttribute('sku', $sku);
                                if (in_array($sku, $conditionsSku, true)) {
                                    $currentQuote = Mage::getSingleton('checkout/session')->getQuote();
                                    if ($currentQuote->hasProductId($product->getId())) {
                                        return false;
                                    }
                                    $quote->addProduct($product, $qty);
                                    $quote->collectTotals()->save();
                                }
                            }
                        } else {
                            $product = Mage::getModel('catalog/product')->setStoreId($storeId)->loadByAttribute('sku', $productSku);
                            if (in_array($productSku, $conditionsSku, true)) {
                                $currentQuote = Mage::getSingleton('checkout/session')->getQuote();
                                if ($currentQuote->hasProductId($product->getId())) {
                                    return false;
                                }
                                $quote->addProduct($product, $qty);
                                $quote->collectTotals()->save();
                            }
                        }
                    } catch (Exception $e) {
                        $this->helper->debug('ERROR:' . $e->getMessage());
                    }
                }
            }
        } catch (Exception $e) {
            $this->helper->debug('ERROR:' . $e->getMessage());
        }
        return;
    }

    public function refillCart()
    {
        if (Mage::getSingleton('customer/session')->isLoggedIn()) {
            return false;
        }

        try {
            $trackingSesssionId = $this->helper->getFetchPointsByKey('_utid');
            $model = Mage::getModel('tracking/log');
            $model->load($trackingSesssionId, 'tracking_id');
            $quoteId = $model->getQuoteId();
            // update timestamp for the user to check user active.
            if($quoteId){
                $model->setDateEntered(gmdate('Y-m-d'))->save();
            }

            if($quoteId){
                $checkout = Mage::getSingleton('checkout/session');
                $coreSession = Mage::getSingleton('core/session');
                // Initialize the cart and 
                $cart = Mage::getModel('checkout/cart');
                $chkQuote = $cart->getQuote();

                $currentQuoteId = $checkout->getQuoteId();

                /*if(!$chkQuote->getId()){
                   $checkout->setQuoteMerged(true);
                }*/

                $quote = Mage::getModel('sales/quote')->load($quoteId);

                if ($quote && $quote->getId() && $quote->getIsActive() && $currentQuoteId === null && (($checkout->getQuoteMerged() === null) ||
                        $checkout->getQuoteMerged() !== true)
                ) {
                    $quoteItems = $quote->getAllVisibleItems();
                
                    $i = 0;
                    foreach ($quoteItems as $item) {
                        $product = Mage::getModel('catalog/product')->load($item->getProductId());

                        $ptype = $item->getProductType();
                        switch ($ptype) {
                            case 'configurable':
                                $customOptions = $item->getProduct()->getTypeInstance(true)
                                    ->getOrderOptions($item->getProduct());
                                $superAttributeInfo = $customOptions['info_buyRequest'];
                                $params = array('qty' => $quoteItems[$i]['qty'],
                                    'super_attribute' => $superAttributeInfo['super_attribute']);
                                $cart->addProduct($product, $params);
                                break;
                            case 'grouped':
                                $productOption = $item->getProduct()->getTypeInstance(true)
                                    ->getOrderOptions($item->getProduct());
                                $childProductId = $productOption['super_product_config']['product_id'];
                                $childQty = array($childProductId => $quoteItems[$i]['qty']);
                                $params = array('super_group' => $childQty);
                                $cart->addProduct($product, $params);
                                break;

                            case 'bundle':
                                $customOptions = $item->getProduct()->getTypeInstance(true)
                                    ->getOrderOptions($item->getProduct());
                                $superAttributeInfo = $customOptions['info_buyRequest'];
                                $params = array('qty' => $quoteItems[$i]['qty'],
                                                'bundle_option' => $superAttributeInfo['bundle_option'],
                                                'bundle_option_qty' => $superAttributeInfo['bundle_option_qty']);
                                $cart->addProduct($product, $params);
                                break;
                            default:
                                $customOptions = $item->getProduct()->getTypeInstance(true)
                                    ->getOrderOptions($item->getProduct());
                                if ($customOptions) {
                                    $superAttributeInfo = $customOptions['info_buyRequest'];
                                    if(isset($superAttributeInfo['options'])){
                                        $superOption = $superAttributeInfo['options'];
                                    }
                                    $params = array('qty' => $quoteItems[$i]['qty'],
                                        'options' => $superOption);
                                } else {
                                    $params = array('qty' => $quoteItems[$i]['qty']);
                                }

                                $cart->addProduct($product, $params);
                                break;
                        }

                        $i++;
                    }

                    $cart->save();
                    $checkout->setQuoteMerged(true);
                }
            }
        } catch (Exception $e) {
            $this->helper->debug('ERROR: ' . $e->getMessage());
        }
    }

     /**
     * set wishlist product data in session.
     *
     * @param Varien_Event_Observer $observer
    */
    public function setWhislistItemToSession(Varien_Event_Observer $observer)
    {
    	Mage::getSingleton('core/session')->setTbWishlistStatus(true);
        return;
    }

     /**
     * To push add to cart data via wishlist only
     *
     * @param Varien_Event_Observer $observer
    */
    public function pushAddToCartByWishlist(Varien_Event_Observer $observer)
    {
        if(!Mage::getSingleton('core/session')->getTbWishlistStatus()){
            return;
        }

       if (!$this->helper->canTrackPages(Targetbay_Tracking_Helper_Data::ADDTOCART)) {
            return false;
        }

        Mage::getSingleton('checkout/session')->setTitle('Shopping cart');

        $productInfo = $observer->getProduct();

        $productEventInfo = $observer->getEvent()->getProduct();

        $quote = Mage::getModel('checkout/cart')->getQuote();
        $item = $quote->getItemByProduct($productEventInfo);
        $productQty = $observer->getEvent()->getProduct()->getQty();
        if ($productInfo->getId()) {
            try {
                switch ($productInfo->getData('type_id')) {
                    case 'grouped':
                        $productIds = Mage::app()->getRequest()->getParam('super_group');

                        $dataItem = array();
                        $productData = array();
                        foreach ($productIds as $id => $qty) {
                            if ($qty < 1) {
                                continue;
                            }
                            $product = Mage::getModel('catalog/product')->load($id);
                            $productData['type'] = $product->getTypeId();
                            $productData['parent_id'] = Mage::app()->getRequest()->getParam('product');
                            $productData['product_id'] = $id;
                            $productData['product_sku'] = $product->getSku();
                            $productData['product_name'] = Mage::helper('core')->escapeHtml($product->getName());
                            $productData['price'] = $product->getFinalPrice() * $qty;
                            $productData['special_price'] = $product->getSpecialPrice();
                            $productData['productimg'] = $this->helper->getImageUrl($product, 'image');
                            $productData['category'] = $this->helper->getProductCategory($product);
                            $productData['category_name'] = $this->helper->getProductCategoryName($product);
                            $productData['quantity'] = $qty;
                            $productData['page_url'] = Mage::getUrl($product->getUrlPath());
                            $productData['attributes'] = '';
                            $dataItem[] = $productData;
                        }

                        $data = $this->helper->getCartInfo();
                        $data['product_type'] = $productInfo->getData('type_id');
                        $data['cart_item'] = $dataItem;
                        break;
                    default:
                        $data = array_merge($this->helper->getCartInfo(), $this->helper->getItemInfo($item, Targetbay_Tracking_Helper_Data::ADDTOCART,$productQty));
                        $data['product_type'] = $item->getProductType();
                        if($item->getProductType() == 'bundle'){
                        	$data['price'] = $item->getProduct()->getFinalPrice() * $item->getProduct()->getQty();
                        }else{
                        	$data['price'] = $item->getPrice() ? $item->getPrice() : $productInfo->getData('price');
                        }

                        if ($customOptions = $this->helper->getCustomOptionsInfo($item, null)) {
                            $data['attributes'] = $customOptions;
                        }
                        break;
                }

                $this->pushPages($data, Targetbay_Tracking_Helper_Data::ADDTOCART);
                if (!Mage::getSingleton('customer/session')->isLoggedIn()) {
                    Mage::getModel('tracking/log')->saveCartData();
                }
            } catch (Exception $e) {
                $this->helper->debug('ERROR: ' . $e->getMessage());
            }
        }
        Mage::getSingleton('core/session')->setTbWishlistStatus(false);
        return;
    }

    /**
     * Used to set UTM data
     **/
    public function setUtmData(){
	$request = Mage::app()->getRequest();
        $utmSource = $request->getParam('utm_source');
        $utmToken = $request->getParam('utm_token');
        $utmMedium = $request->getParam('utm_medium');
        $utmCampaign = $request->getParam('utm_campaign');

        // Set Token Values
        if ($utmSource !== '' && $utmSource !== NULL) {
            $this->helper->cookie->delete('utm_source');
            $this->helper->cookie->set('utm_source', $utmSource, null, null, null, null, false);
        }

        if ($utmToken !== '' && $utmToken !== NULL) {
            $this->helper->cookie->delete('utm_token');
            $this->helper->cookie->set('utm_token', $utmToken, null, null, null, null, false);
        }

        if ($utmMedium !== '' && $utmMedium !== NULL) {
            $this->helper->cookie->delete('utm_medium');
            $this->helper->cookie->set('utm_medium', $utmMedium, null, null, null, null, false);
        }

        if ($utmCampaign !== '' && $utmCampaign !== NULL) {
            $this->helper->cookie->delete('utm_campaign');
            $this->helper->cookie->set('utm_campaign', $utmCampaign, null, null, null, null, false);
        }

	}
}
