<?php

/*
 * @author Targetbay
 * @copyright 2016 Sathishkumar Mariappan <sathishkumar.m@innoppl.com>
 * @package Targetbay_Tracking
 */

class Targetbay_Tracking_Model_Reviewdata extends Mage_Customer_Model_Customer
{
    /**
     * Get the TargetBay reviews and rating for product.
     *
     * @return void|bool
     */
    public function getProductReview()
    {
        if (!Mage::helper('tracking')->getGoogleProductReviewSnippet()) {
            return false;
        }

        $mostViewedProducts = $this->getMostViewedProducts();
        $data = array();

        if (count($mostViewedProducts) > 0) {
            $data['product_id'] = implode(',', $mostViewedProducts);
            $cacheTag = 'block_html';

            /*
            $type = Targetbay_Tracking_Helper_Data::RATINGS_STATS;
            $websiteId = Mage::app()->getStore()->getWebsiteId();
            $apiToken = '?api_token=' . Mage::helper('tracking')->getApiToken($websiteId);
            $feedUrl = Mage::helper('tracking')->getHostname() . $type . $apiToken;
            $data['index_name'] = Mage::helper('tracking')->getApiIndex($websiteId);
            $data['qaDisplay'] = Mage::helper('tracking')->getGoogleQuestionReviewSnippet();
            */

            $helper = Mage::helper('tracking');
            $data['qaDisplay'] = $helper->getGoogleQuestionReviewSnippet();
            $feedUrl = $helper->getHost(Targetbay_Tracking_Helper_Data::RATINGS_STATS, Mage::app()->getStore()->getWebsiteId());

            $response = $helper->postPageInfo($feedUrl, json_encode($data));
            $responseBody = json_decode($response);

            $cache = Mage::app()->getCache();
            foreach ($mostViewedProducts as $productId) {
                $cacheResponseId = 'product_tbreview_response_' . $productId;
                //$cacheCountId = 'product_review_count_' . $productId;
                //$cacheTbCountId = 'product_tbreview_count_' . $productId;

                if (!empty($responseBody->$productId)) {
                    $productReview = $responseBody->$productId;
                    if ($productReview->totalReviewsCount > 0) {
                        $cache->remove($cacheResponseId);
                        $cache->save(serialize($productReview), $cacheResponseId, array($cacheTag));
                    }
                }
            }
        }
    }

    /**
     * Get the TargetBay review count and rating for product.
     *
     * @return void|bool
     */
    public function getSiteReview()
    {
        if (!Mage::helper('tracking')->getGoogleSiteReviewSnippet()) {
            return false;
        }

        $cacheResponseId = 'site_tbreview_response';
        $cacheTag = 'block_html';
        $data = array();

        /*
        $websiteId = Mage::app()->getStore()->getWebsiteId();
        $type = Targetbay_Tracking_Helper_Data::RATINGS_STATS;
        $apiToken = '?api_token=' . Mage::helper('tracking')->getApiToken($websiteId);
        $feedUrl = Mage::helper('tracking')->getHostname() . $type . $apiToken;
        $data['index_name'] = Mage::helper('tracking')->getApiIndex($websiteId);
        */

        $helper = Mage::helper('tracking');
        $feedUrl = $helper->getHost(Targetbay_Tracking_Helper_Data::RATINGS_STATS, Mage::app()->getStore()->getWebsiteId());

        $response = $helper->postPageInfo($feedUrl, json_encode($data));
        $responseBody = json_decode($response);

        if (!empty($responseBody) && $responseBody->reviews_count > 0) {
            $cache = Mage::app()->getCache();
            $cache->remove($cacheResponseId);
            $cache->save(serialize($responseBody), $cacheResponseId, array($cacheTag));
        }
    }

    /**
     * Get most viewed product of 30 days.
     *
     * @return array
     */
    public function getMostViewedProducts()
    {
        // Number of products to display.
        $productCount = 50;

        // store ID
        $storeId    = Mage::app()->getStore()->getId();

        // Get today and last 30 days time.
        $today = time();
        $last = $today - (60 * 60 * 24 * 30);

        $from = date('Y-m-d', $last);
        $to = date('Y-m-d', $today);

        // Get most viewed products for last 30 days.
        $productCollection = Mage::getResourceModel('reports/product_collection')
            ->addAttributeToSelect('entity_id')
            ->setStoreId($storeId)
            ->addStoreFilter($storeId)
            ->addViewsCount()
            ->addViewsCount($from, $to)
            ->setPageSize($productCount);

        Mage::getSingleton('catalog/product_status')
            ->addVisibleFilterToCollection($productCollection);
        Mage::getSingleton('catalog/product_visibility')
            ->addVisibleInCatalogFilterToCollection($productCollection);

        $products = array();
        foreach ($productCollection as $product) {
            $products[] = $product->getEntityId();
        }

        return $products;
    }
}
