<?php

/*
 * @author TargetBay
 * @copyright 2016 Sathishkumar Mariappan <sathishkumar.m@innoppl.com>
 * @package Targetbay_Tracking
 */

class Targetbay_Tracking_IndexController extends Mage_Core_Controller_Front_Action
{
    /**
     * Used to refill or merge abandoned cart with current cart
     * @return mixed
     */
    public function reloadAction()
    {
         //validate quote id param
        $abandonedQuoteId = Mage::app()->getRequest()->getParam('quote_id');
        if(!$abandonedQuoteId || $abandonedQuoteId === NULL){
           $this->_redirectAfterReload();
           return;
        }

        $checkout = Mage::getSingleton('checkout/session');
        $helper = Mage::helper('tracking');

        //Redirect when items already added for this session
        if ($checkout->getAbandonedMerged()) {
            return $this->_redirectAfterReload();
        }

        //add targetbay session id cookie
        $guestUserId = Mage::app()->getRequest()->getParam('guest_user_id');
        
        if ($guestUserId != '' && !Mage::getSingleton('customer/session')->isLoggedIn()) {
                Mage::getSingleton('core/session')->setTrackingSessionId($guestUserId);
                $helper->updateFetchPoints('_utid', $guestUserId);
                $helper->updateFetchPoints('_usid', $guestUserId);
        }
        
        try {

            //prepare abandoned quote
            $abandonedQuote = Mage::getModel('sales/quote')->load($abandonedQuoteId);
            if(!$abandonedQuote->getId() && !$abandonedQuote->getIsActive()){
                return $this->_redirectAfterReload();
            }

            //update current quote with abandoned quote
            $currentQuote = $checkout->getQuote();
            if($currentQuote && $currentQuote->getId()){
                $currentQuote = Mage::getModel('sales/quote')->load($currentQuote->getId());
                //compare and remove same items from current quote
                $filteredAbandonedQuote = $this->_filterAbandonedQuoteMergeBefore($abandonedQuote, $currentQuote);
                $currentQuote->merge($filteredAbandonedQuote);
                $currentQuote->collectTotals()->save();
            } else {
                //update abandoned quote to customer
                $cart = Mage::getModel('checkout/cart');
                if(Mage::getSingleton('customer/session')->isLoggedIn()){
                    $cart->setQuote($abandonedQuote);
                    $cart->getQuote()->assignCustomer(Mage::getSingleton('customer/session')->getCustomer());
                } else {
                    //add new quote to not loggedin  user
                    Mage::getSingleton('core/session')->setAbandonedMail(true);
                    $quoteItems = $abandonedQuote->getAllVisibleItems();
                    $i = 0;
                    foreach ($quoteItems as $item) {
                        $product = Mage::getModel('catalog/product')->load($item->getProductId());
                        $productType = $item->getProductType();

                        switch ($productType) {
                            case 'configurable':
                                $customOptions = $item->getProduct()->getTypeInstance(true)
                                    ->getOrderOptions($item->getProduct());
                                $superAttributeInfo = $customOptions['info_buyRequest'];
                                $params = array('qty' => $quoteItems[$i]['qty'],
                                    'super_attribute' => $superAttributeInfo['super_attribute']);
                                $cart->addProduct($product, $params);
                                break;
                            case 'grouped':
                                $productOption = $item->getProduct()->getTypeInstance(true)
                                    ->getOrderOptions($item->getProduct());
                                $childProductId = $productOption['super_product_config']['product_id'];
                                $childQty = array($childProductId => $quoteItems[$i]['qty']);
                                $params = array('super_group' => $childQty);
                                $cart->addProduct($product, $params);
                                break;
                            case 'bundle':
                                $customOptions = $item->getProduct()->getTypeInstance(true)
                                    ->getOrderOptions($item->getProduct());
                                $superAttributeInfo = $customOptions['info_buyRequest'];
                                $params = array('qty' => $quoteItems[$i]['qty'],
                                                'bundle_option' => $superAttributeInfo['bundle_option'],
                                                'bundle_option_qty' => $superAttributeInfo['bundle_option_qty']);
                                $cart->addProduct($product, $params);
                                break;
                            default:
                                $customOptions = $item->getProduct()->getTypeInstance(true)
                                    ->getOrderOptions($item->getProduct());
                                if ($customOptions) {
                                    $superAttributeInfo = $customOptions['info_buyRequest'];
                                    $params = array('qty' => $quoteItems[$i]['qty'],
                                        'options' => $superAttributeInfo['options']);
                                } else {
                                    $params = array('qty' => $quoteItems[$i]['qty']);
                                }

                                $cart->addProduct($product, $params);
                                break;
                        }
                        $i++;
                    }

                }
                $cart->save();
            }

            $checkout->setAbandonedMerged(true);

        } catch (Exception $e) {
            Mage::getSingleton('checkout/session')
                ->addError(Mage::helper('core')->escapeHtml($e->getMessage()));
            $helper->debug('ERROR: ' . $e->getMessage());
        }

        return $this->_redirectAfterReload();
    }

    /**
     * Used to redirect with utm params to checkout cart page
     * @return mixed
     */
    public function _redirectAfterReload()
    {
        $url = 'checkout/cart/';
        $utmSource = Mage::app()->getRequest()->getParam('utm_source');
        $utmToken = Mage::app()->getRequest()->getParam('utm_token');
        $utmMedium = Mage::app()->getRequest()->getParam('utm_medium');
        $utmSubject = Mage::app()->getRequest()->getParam('utm_campaign');

        return $this->_redirect(
            $url,
            array('_secure' => Mage::app()->getStore()->isCurrentlySecure(),'_query'=>array(
                'utm_source' => $utmSource,
                'utm_token' => $utmToken,
                'utm_medium' => $utmMedium,
                'utm_campaign'=> $utmSubject
            ))
        );
    }

    /**
     * @return mixed
     */
    public function tbaddtobagAction()
    {
        try {
            $productData = base64_decode($this->getRequest()->getParam('product_id'));
            $staticArrayDecode = json_decode($productData, true);
            $productIds = $staticArrayDecode;
            $storeId = Mage::app()->getStore()->getId();
            $qty = 1;

            if (count($staticArrayDecode) < 1) {
                $this->_redirectAfterReload();
            }

            $helper = Mage::helper('tracking');
            $cart = Mage::helper('checkout/cart')->getCart();
            $cart->init();
            $bundleOptionQty = array();

            foreach ($staticArrayDecode as $key => $value) {
                $productId = $key;
                $productIds = $value;

                if (count($productIds) < 1) {
                    return false;
                }

                foreach ($productIds as $pId) {
                    $childProductId = $pId;

                    $product = Mage::getModel('catalog/product')
                                                ->setStoreId($storeId)
                                                ->load($productId);

                    $productType = $product->getTypeId();

                    switch ($productType) {
                        case Mage_Catalog_Model_Product_Type::TYPE_CONFIGURABLE:
                            $productOptions = Mage::helper('tracking/inventary')
                                                ->getConfigurableProductOptionIds($productId, $childProductId);

                            $params = array('qty' => $qty,
                                            'super_attribute' =>  $productOptions);
                            break;
                        case Mage_Catalog_Model_Product_Type::TYPE_GROUPED:
                            $option = array($childProductId => $qty);
                            $params = array('super_group' =>  $option);
                            break;
                        case Mage_Catalog_Model_Product_Type::TYPE_BUNDLE:
                            $bundleOption = Mage::helper('tracking/inventary')
                                                ->getBundledProductOptionIds($productId, $childProductId);

                            foreach ($bundleOption as $bundleKey => $bundleValue) {
                                $bundleOptionQty[$bundleKey] = $qty;
                            }

                            $params = array('bundle_option' => $bundleOption,
                                            'bundle_option_qty' => $bundleOptionQty,
                                            'qty' => $qty);
                            break;
                        default:
                            $params = array('qty' => $qty);
                            break;
                    }

                    $cart->addProduct($product, $params);
                }
            }

            $cart->save();
            Mage::getSingleton('checkout/session')->setCartWasUpdated(true);
        } catch (Exception $e) {
            $helper->debug('ERROR: ' . $e->getMessage());
        }

        return $this->_redirectAfterReload();
    }

    /**
    * Used to remove same items from current quote
    * @param Mage_Sales_Model_Quote abandonedQuote
    * @param Mage_Sales_Model_Quote currentQuote
    * @return Mage_Sales_Model_Quote abandonedQuote
    **/
    protected function _filterAbandonedQuoteMergeBefore(Mage_Sales_Model_Quote $abandonedQuote, Mage_Sales_Model_Quote $currentQuote){
        foreach ($currentQuote->getAllVisibleItems() as $currentitem) {
            foreach ($abandonedQuote->getAllItems() as $abandonedItem) {
                if ($abandonedItem->getProductId() == $currentitem->getProductId()) {
                    $currentitem->isDeleted(true);
                    if ($currentitem->getHasChildren()) {
                        foreach ($currentitem->getChildren() as $child) {
                            $child->isDeleted(true);
                        }
                    }
                }
            }

        }
        return $abandonedQuote;
    }
}
